@@ -637,7 +637,8 @@ void ainfo_page(void){
 */
 void attachment_list(
   const char *zTarget,   /* Object that things are attached to */
-  const char *zHeader    /* Header to display with attachments */
+  const char *zHeader,   /* Header to display with attachments */
+  int fHorizontalRule    /* Insert <hr> separator above header */
 ){
   int cnt = 0;
   Stmt q;
@@ -657,7 +658,12 @@ void attachment_list(
     const char *zSrc = db_column_text(&q, 4);
     const char *zDispUser = zUser && zUser[0] ? zUser : "anonymous";
     if( cnt==0 ){
-      cgi_printf("%s\n",(zHeader));
+      cgi_printf("<section class='attachlist'>\n");
+      if( fHorizontalRule ){
+        cgi_printf("<hr>\n");
+      }
+      cgi_printf("%s\n"
+             "<ul>\n",(zHeader));
     }
     cnt++;
     cgi_printf("<li>\n"
@@ -669,7 +675,8 @@ void attachment_list(
            "</li>\n",(href("%R/ainfo/%!S",zUuid)));
   }
   if( cnt ){
-    cgi_printf("</ul>\n");
+    cgi_printf("</ul>\n"
+           "</section>\n");
   }
   db_finalize(&q);
 
