@@ -40,9 +40,9 @@ void bisect_path(void){
   bisect.bad = db_lget_int("bisect-bad", 0);
   bisect.good = db_lget_int("bisect-good", 0);
   if( bisect.good>0 && bisect.bad==0 ){
-    path_shortest(bisect.good, bisect.good, 0, 0, 0);
+    path_shortest(bisect.good, bisect.good, 0, 0, 0, 0);
   }else if( bisect.bad>0 && bisect.good==0 ){
-    path_shortest(bisect.bad, bisect.bad, 0, 0, 0);
+    path_shortest(bisect.bad, bisect.bad, 0, 0, 0, 0);
   }else if( bisect.bad==0 && bisect.good==0 ){
     fossil_fatal("neither \"good\" nor \"bad\" versions have been identified");
   }else{
@@ -58,7 +58,7 @@ void bisect_path(void){
       }
     }
     blob_reset(&log);
-    p = path_shortest(bisect.good, bisect.bad, bDirect, 0, &skip);
+    p = path_shortest(bisect.good, bisect.bad, bDirect, 0, &skip, 0);
     bag_clear(&skip);
     if( p==0 ){
       char *zBad = db_text(0,"SELECT uuid FROM blob WHERE rid=%d",bisect.bad);
@@ -201,6 +201,22 @@ static void bisect_append_skip(int rid){
   );
 }
 
+/*
+** Append a VALUES entry to the bilog table insert
+*/
+static void bisect_log_append(Blob *pSql,int iSeq,const char *zStat,int iRid){
+  if( (iSeq%6)==3 ){
+    blob_append_sql(pSql, ",\n  ");
+  }else if( iSeq>1 ){
+    blob_append_sql(pSql, ",");
+  }
+  if( zStat ){
+    blob_append_sql(pSql, "(%d,%Q,%d)", iSeq, zStat, iRid);
+  }else{
+    blob_append_sql(pSql, "(NULL,NULL,%d)", iRid);
+  }
+}
+
 /*
 ** Create a TEMP table named "bilog" that contains the complete history
 ** of the current bisect.
@@ -218,10 +234,10 @@ static void bisect_append_skip(int rid){
 int bisect_create_bilog_table(int iCurrent, const char *zDesc, int bDetail){
   char *zLog;
   Blob log, id;
-  Stmt q;
   int cnt = 0;
   int lastGood = -1;
   int lastBad = -1;
+  Blob ins = BLOB_INITIALIZER;
 
   if( zDesc!=0 ){
     blob_init(&log, 0, 0);
@@ -256,51 +272,38 @@ int bisect_create_bilog_table(int iCurrent, const char *zDesc, int bDetail){
      "  seq INTEGER UNIQUE"        /* Check-in number */
      ");"
   );
-  db_prepare(&q, "INSERT OR IGNORE INTO bilog(seq,stat,rid)"
-                 " VALUES(:seq,:stat,:rid)");
+  blob_append_sql(&ins, "INSERT OR IGNORE INTO bilog(seq,stat,rid) VALUES");
   while( blob_token(&log, &id) ){
     int rid;
-    db_bind_int(&q, ":seq", ++cnt);
+    cnt++;
     if( blob_str(&id)[0]=='s' ){
       rid = atoi(blob_str(&id)+1);
-      db_bind_text(&q, ":stat", "SKIP");
-      db_bind_int(&q, ":rid", rid);
+      bisect_log_append(&ins, cnt, "SKIP", rid);
     }else{
       rid = atoi(blob_str(&id));
       if( rid>0 ){
-        db_bind_text(&q, ":stat","GOOD");
-        db_bind_int(&q, ":rid", rid);
+        bisect_log_append(&ins, cnt, "GOOD", rid);
         lastGood = rid;
       }else{
-        db_bind_text(&q, ":stat", "BAD");
-        db_bind_int(&q, ":rid", -rid);
+        bisect_log_append(&ins, cnt, "BAD", -rid);
         lastBad = -rid;
       }
     }
-    db_step(&q);
-    db_reset(&q);
   }
   if( iCurrent>0 ){
-    db_bind_int(&q, ":seq", ++cnt);
-    db_bind_text(&q, ":stat", "CURRENT");
-    db_bind_int(&q, ":rid", iCurrent);
-    db_step(&q);
-    db_reset(&q);
+    bisect_log_append(&ins, ++cnt, "CURRENT", iCurrent);
   }
   if( bDetail && lastGood>0 && lastBad>0 ){
     PathNode *p;
-    p = path_shortest(lastGood, lastBad, bisect_option("direct-only"),0, 0);
+    p = path_shortest(lastGood, lastBad, bisect_option("direct-only"),0, 0, 0);
     while( p ){
-      db_bind_null(&q, ":seq");
-      db_bind_null(&q, ":stat");
-      db_bind_int(&q, ":rid", p->rid);
-      db_step(&q);
-      db_reset(&q);
+      bisect_log_append(&ins, ++cnt, 0, p->rid);
       p = p->u.pTo;
     }
     path_reset();
   }
-  db_finalize(&q);
+  db_exec_sql(blob_sql_text(&ins));
+  blob_reset(&ins);
   return 1;
 }
 
@@ -336,7 +339,7 @@ char *bisect_permalink(void){
     if( blob_size(&link)>0 ) blob_append(&link, "-", 1);
     blob_appendf(&link, "%c%.10s", cPrefix, zUuid);
   }
-  zResult = mprintf("%s", blob_str(&link));
+  zResult = fossil_strdup(blob_str(&link));
   blob_reset(&link);
   blob_reset(&log);
   blob_reset(&id);
@@ -546,10 +549,12 @@ static void bisect_run(void){
 **
 **       List the versions in between the inner-most "bad" and "good".
 **
-** > fossil bisect ui
+** > fossil bisect ui ?HOST@USER:PATH?
 **
 **       Like "fossil ui" except start on a timeline that shows only the
-**       check-ins that are part of the current bisect.
+**       check-ins that are part of the current bisect.  If the optional
+**       fourth term is added, then information is shown for the bisect that
+**       occurred in the PATH directory by USER on remote machine HOST.
 **
 ** > fossil bisect undo
 **
@@ -722,13 +727,19 @@ void bisect_cmd(void){
     bisect_reset();
   }else if( strcmp(zCmd, "ui")==0 ){
     char *newArgv[8];
+    verify_all_options();
     newArgv[0] = g.argv[0];
     newArgv[1] = "ui";
     newArgv[2] = "--page";
     newArgv[3] = "timeline?bisect";
-    newArgv[4] = 0;
+    if( g.argc==4 ){
+      newArgv[4] = g.argv[3];
+      g.argc = 5;
+    }else{
+      g.argc = 4;
+    }
+    newArgv[g.argc] = 0;
     g.argv = newArgv;
-    g.argc = 4;
     cmd_webserver();
   }else if( strncmp(zCmd, "vlist", n)==0
          || strncmp(zCmd, "ls", n)==0
