@@ -601,7 +601,7 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 ** Run various subcommands to manage branches of the open repository or
 ** of the repository identified by the -R or --repository option.
 **
-** >  fossil branch close|reopen ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
+** > fossil branch close|reopen ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
 **
 **       Adds or cancels the "closed" tag to one or more branches.
 **       It accepts arbitrary unambiguous symbolic names but
@@ -615,22 +615,22 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 **         --date-override DATE  DATE to use instead of 'now'
 **         --user-override USER  USER to use instead of the current default
 **
-** >  fossil branch current
+** > fossil branch current
 **
 **        Print the name of the branch for the current check-out
 **
-** >  fossil branch hide|unhide ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
+** > fossil branch hide|unhide ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
 **
 **       Adds or cancels the "hidden" tag for the specified branches or
 **       or check-in IDs. Accepts the same options as the close
 **       subcommand.
 **
-** >  fossil branch info BRANCH-NAME
+** > fossil branch info BRANCH-NAME
 **
 **        Print information about a branch
 **
-** >  fossil branch list|ls ?OPTIONS? ?GLOB?
-** >  fossil branch lsh ?OPTIONS? ?LIMIT?
+** > fossil branch list|ls ?OPTIONS? ?GLOB?
+** > fossil branch lsh ?OPTIONS? ?LIMIT?
 **
 **        List all branches.
 **
@@ -643,8 +643,8 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 **          -r               Reverse the sort order
 **          -t               Show recently changed branches first
 **          --self           List only branches where you participate
-**          --username USER  List only branches where USER participate
-**          --users N        List up to N users partipiating
+**          --username USER  List only branches where USER participates
+**          --users N        List up to N users participating
 **
 **        The current branch is marked with an asterisk.  Private branches are
 **        marked with a hash sign.
@@ -656,10 +656,16 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 **        but no GLOB argument.  All other options are supported, with -t being
 **        an implied no-op.
 **
-** >  fossil branch new BRANCH-NAME BASIS ?OPTIONS?
+** > fossil branch new BRANCH-NAME BASIS ?OPTIONS?
 **
 **        Create a new branch BRANCH-NAME off of check-in BASIS.
 **
+**        This command is available for people who want to create a branch
+**        in advance.  But  the use of this command is discouraged.  The
+**        preferred idiom in Fossil is to create new branches at the point
+**        of need, using the "--branch NAME" option to the "fossil commit"
+**        command.
+**
 **        Options:
 **          --private             Branch is private (i.e., remains local)
 **          --bgcolor COLOR       Use COLOR instead of automatic background
@@ -669,12 +675,6 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 **          --date-override DATE  DATE to use instead of 'now'
 **          --user-override USER  USER to use instead of the current default
 **
-**        DATE may be "now" or "YYYY-MM-DDTHH:MM:SS.SSS". If in
-**        year-month-day form, it may be truncated, the "T" may be
-**        replaced by a space, and it may also name a timezone offset
-**        from UTC as "-HH:MM" (westward) or "+HH:MM" (eastward).
-**        Either no timezone suffix or "Z" means UTC.
-**
 ** Options:
 **    -R|--repository REPO       Run commands on repository REPO
 */
@@ -873,7 +873,8 @@ static void new_brlist_page(void){
     char *zAge = human_readable_age(rNow - rMtime);
     sqlite3_int64 iMtime = (sqlite3_int64)(rMtime*86400.0);
     if( zMergeTo && zMergeTo[0]==0 ) zMergeTo = 0;
-    if( zBgClr == 0 ){
+    if( zBgClr ) zBgClr = reasonable_bg_color(zBgClr, 0);
+    if( zBgClr==0 ){
       if( zBranch==0 || strcmp(zBranch,"trunk")==0 ){
         zBgClr = 0;
       }else{
