@@ -208,7 +208,7 @@ void page_dir(void){
       isSymbolicCI = (sqlite3_strnicmp(zUuid, zCI, strlen(zCI))!=0);
       isBranchCI = branch_includes_uuid(zCI, zUuid);
       if( bDocDir ) zCI = mprintf("%S", zUuid);
-      Th_Store("current_checkin", zCI);
+      Th_StoreUnsafe("current_checkin", zCI);
     }else{
       zCI = 0;
     }
@@ -750,7 +750,7 @@ void page_tree(void){
   /* If a regular expression is specified, compile it */
   zRE = P("re");
   if( zRE ){
-    re_compile(&pRE, zRE, 0);
+    fossil_re_compile(&pRE, zRE, 0);
     zREx = mprintf("&re=%T", zRE);
   }
   cgi_check_for_malice();
@@ -774,7 +774,7 @@ void page_tree(void){
                          " FROM event WHERE objid=%d", rid);
       isSymbolicCI = (sqlite3_strnicmp(zUuid, zCI, strlen(zCI)) != 0);
       isBranchCI = branch_includes_uuid(zCI, zUuid);
-      Th_Store("current_checkin", zCI);
+      Th_StoreUnsafe("current_checkin", zCI);
     }else{
       zCI = 0;
     }
@@ -1041,7 +1041,8 @@ static const char zComputeFileAgeSetup[] =
 "  fid INTEGER,\n"
 "  mid INTEGER,\n"
 "  mtime DATETIME,\n"
-"  pathname TEXT\n"
+"  pathname TEXT,\n"
+"  uuid TEXT\n"
 ");\n"
 "CREATE VIRTUAL TABLE IF NOT EXISTS temp.foci USING files_of_checkin;\n"
 ;
@@ -1053,8 +1054,9 @@ static const char zComputeFileAgeRun[] =
 "             SELECT plink.pid\n"
 "               FROM ckin, plink\n"
 "              WHERE plink.cid=ckin.x)\n"
-"INSERT OR IGNORE INTO fileage(fnid, fid, mid, mtime, pathname)\n"
-"  SELECT filename.fnid, mlink.fid, mlink.mid, event.mtime, filename.name\n"
+"INSERT OR IGNORE INTO fileage(fnid, fid, mid, mtime, pathname, uuid)\n"
+"  SELECT filename.fnid, mlink.fid, mlink.mid, event.mtime, filename.name,\n"
+"         foci.uuid\n"
 "    FROM foci, filename, blob, mlink, event\n"
 "   WHERE foci.checkinID=:ckin\n"
 "     AND foci.filename GLOB :glob\n"
@@ -1165,7 +1167,6 @@ void fileage_page(void){
   double baseTime;
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  if( exclude_spiders(0) ) return;
   zName = P("name");
   if( zName==0 ) zName = "tip";
   rid = symbolic_name_to_rid(zName, "ci");
