@@ -747,11 +747,11 @@ void captcha_test(void){
            "to see how HEX would be rendered in the current captcha font.\n"
            "<h2>Debug/Testing Values:</h2>\n"
            "<ul>\n"
-           "<li> g.isHuman = %d\n"
+           "<li> g.isRobot = %d\n"
            "<li> g.zLogin = %h\n"
            "<li> login_cookie_welformed() = %d\n"
            "<li> captcha_is_correct(1) = %d.\n"
-           "</ul>\n",(g.isHuman),(g.zLogin),(login_cookie_wellformed()),(captcha_is_correct(1)));
+           "</ul>\n",(g.isRobot),(g.zLogin),(login_cookie_wellformed()),(captcha_is_correct(1)));
     style_finish_page();
   }else{
     style_set_current_feature("test");
@@ -763,6 +763,14 @@ void captcha_test(void){
   }
 }
 
+/*
+** WEBPAGE: honeypot
+** This page is a honeypot for spiders and bots.
+*/
+void honeypot_page(void){
+  (void)exclude_spiders(0);
+}
+
 /*
 ** Check to see if the current request is coming from an agent that 
 ** self-identifies as a spider.
@@ -779,7 +787,6 @@ void captcha_test(void){
 */
 int exclude_spiders(int bTest){
   if( !bTest ){
-    if( g.isHuman ) return 0;  /* This user has already proven human */
     if( g.zLogin!=0 ) return 0;  /* Logged in.  Consider them human */
     if( login_cookie_wellformed() ){
       /* Logged into another member of the login group */
@@ -789,13 +796,10 @@ int exclude_spiders(int bTest){
 
   /* This appears to be a spider.  Offer the captcha */
   style_set_current_feature("captcha");
-  style_header("I think you are a robot");
+  style_header("Captcha");
   style_submenu_enable(0);
   cgi_printf("<form method='POST' action='%R/ityaar'>\n"
-         "<p>You seem like a robot.\n"
-         "\n"
-         "<p>If you are human, you can prove that by solving the captcha below,\n"
-         "after which you will be allowed to proceed.\n");
+         "<h2>Prove that you are human:\n");
   if( bTest ){
     cgi_printf("<input type=\"hidden\" name=\"istest\" value=\"1\">\n");
   }
@@ -833,7 +837,7 @@ void captcha_callback(void){
     }
     login_redirect_to_g();
   }else{
-    g.isHuman = 0;
+    g.isRobot = 1;
     (void)exclude_spiders(bTest);
     if( bTest ){
       cgi_printf("<hr><p>Wrong code.  Try again\n");
