@@ -75,6 +75,7 @@
 # include <sys/socket.h>
 # include <sys/un.h>
 # include <netinet/in.h>
+# include <netdb.h>
 # include <arpa/inet.h>
 # include <sys/times.h>
 # include <sys/time.h>
@@ -106,8 +107,8 @@
 #define PB(x)         cgi_parameter_boolean(x)
 #define PCK(x)        cgi_parameter_checked(x,1)
 #define PIF(x,y)      cgi_parameter_checked(x,y)
-#define P_NoBot(x)    cgi_parameter_nosql((x),0)
-#define PD_NoBot(x,y) cgi_parameter_nosql((x),(y))
+#define P_NoBot(x)    cgi_parameter_no_attack((x),0)
+#define PD_NoBot(x,y) cgi_parameter_no_attack((x),(y))
 
 /*
 ** Shortcut for the cgi_printf() routine.  Instead of using the
@@ -243,7 +244,7 @@ char *cgi_extract_content(void){
 /*
 ** Additional information used to form the HTTP reply
 */
-static const char *zContentType = "text/html";   /* Content type of the reply */
+static const char *zReplyMimeType = "text/html"; /* Content type of the reply */
 static const char *zReplyStatus = "OK";          /* Reply status description */
 static int iReplyStatus = 200;               /* Reply status code */
 static Blob extraHeader = BLOB_INITIALIZER;  /* Extra header text */
@@ -258,7 +259,9 @@ static int rangeEnd = 0;                     /* End of Range: plus 1 */
 ** other content type is being returned.
 */
 void cgi_set_content_type(const char *zType){
-  zContentType = fossil_strdup(zType);
+  int i;
+  for(i=0; zType[i]>='+' && zType[i]<='z'; i++){}
+  zReplyMimeType = fossil_strndup(zType, i);
 }
 
 /*
@@ -338,10 +341,10 @@ static int is_gzippable(void){
   /* Maintenance note: this oddball structure is intended to make
   ** adding new mimetypes to this list less of a performance hit than
   ** doing a strcmp/glob over a growing set of compressible types. */
-  switch(zContentType ? *zContentType : 0){
+  switch(zReplyMimeType ? *zReplyMimeType : 0){
     case (int)'a':
-      if(0==fossil_strncmp("application/",zContentType,12)){
-        const char * z = &zContentType[12];
+      if(0==fossil_strncmp("application/",zReplyMimeType,12)){
+        const char * z = &zReplyMimeType[12];
         switch(*z){
           case (int)'j':
             return fossil_strcmp("javascript", z)==0
@@ -356,10 +359,10 @@ static int is_gzippable(void){
       }
       break;
     case (int)'i':
-      return fossil_strcmp(zContentType, "image/svg+xml")==0
-        || fossil_strcmp(zContentType, "image/vnd.microsoft.icon")==0;
+      return fossil_strcmp(zReplyMimeType, "image/svg+xml")==0
+        || fossil_strcmp(zReplyMimeType, "image/vnd.microsoft.icon")==0;
     case (int)'t':
-      return fossil_strncmp(zContentType, "text/", 5)==0;
+      return fossil_strncmp(zReplyMimeType, "text/", 5)==0;
   }
   return 0;
 }
@@ -404,6 +407,8 @@ size_t cgi_fread(void *ptr, size_t nmemb){
   return ssl_read_server(g.httpSSLConn, ptr, nmemb, 1);
 #else
   fossil_fatal("SSL not available");
+  /* NOT REACHED */
+  return 0;
 #endif
 }
 
@@ -451,7 +456,7 @@ static void cgi_fflush(void){
 /*
 ** Given a Content-Type value, returns a string suitable for appending
 ** to the Content-Type header for adding (or not) the "; charset=..."
-** part. It returns an empty string for most types or if zContentType
+** part. It returns an empty string for most types or if zReplyMimeType
 ** is NULL.
 **
 ** See forum post f60dece061c364d1 for the discussions which lead to
@@ -461,8 +466,8 @@ static void cgi_fflush(void){
 ** most types (and not required for many others which may ostensibly
 ** benefit from one, as detailed in that forum post).
 */
-static const char * content_type_charset(const char *zContentType){
-  if(0==fossil_strncmp(zContentType,"text/",5)){
+static const char * content_type_charset(const char *zReplyMimeType){
+  if(0==fossil_strncmp(zReplyMimeType,"text/",5)){
     return "; charset=utf-8";
   }
   return "";
@@ -502,12 +507,12 @@ void cgi_reply(void){
   }
   if( etag_tag()[0]!=0
    && iReplyStatus==200
-   && strcmp(zContentType,"text/html")!=0
+   && strcmp(zReplyMimeType,"text/html")!=0
   ){
     /* Do not cache HTML replies as those will have been generated and
     ** will likely, therefore, contains a nonce and we want that nonce to
     ** be different every time. */
-    blob_appendf(&hdr, "ETag: %s\r\n", etag_tag());
+    blob_appendf(&hdr, "ETag: \"%s\"\r\n", etag_tag());
     blob_appendf(&hdr, "Cache-Control: max-age=%d\r\n", etag_maxage());
     if( etag_mtime()>0 ){
       blob_appendf(&hdr, "Last-Modified: %s\r\n",
@@ -544,9 +549,9 @@ void cgi_reply(void){
   */
 
   if( iReplyStatus!=304 ) {
-    blob_appendf(&hdr, "Content-Type: %s%s\r\n", zContentType,
-                 content_type_charset(zContentType));
-    if( fossil_strcmp(zContentType,"application/x-fossil")==0 ){
+    blob_appendf(&hdr, "Content-Type: %s%s\r\n", zReplyMimeType,
+                 content_type_charset(zReplyMimeType));
+    if( fossil_strcmp(zReplyMimeType,"application/x-fossil")==0 ){
       cgi_combine_header_and_body();
       blob_compress(&cgiContent[0], &cgiContent[0]);
     }
@@ -640,6 +645,9 @@ NORETURN void cgi_redirect_with_status(
   cgi_reply();
   fossil_exit(0);
 }
+NORETURN void cgi_redirect_perm(const char *zURL){
+  cgi_redirect_with_status(zURL, 301, "Moved Permanently");
+}
 NORETURN void cgi_redirect(const char *zURL){
   cgi_redirect_with_status(zURL, 302, "Moved Temporarily");
 }
@@ -691,17 +699,37 @@ const char *cgi_referer(const char *zDefault){
 
 /*
 ** Return true if the current request is coming from the same origin.
+**
+** If the request comes from a different origin and bErrorLog is true, then
+** put a warning message on the error log as this was a possible hack
+** attempt.
 */
-int cgi_same_origin(void){
+int cgi_same_origin(int bErrorLog){
   const char *zRef;
+  char *zToFree = 0;
   int nBase;
+  int rc;
   if( g.zBaseURL==0 ) return 0;
   zRef = P("HTTP_REFERER");
   if( zRef==0 ) return 0;
+  if( strchr(zRef,'%')!=0 ){
+    zToFree = strdup(zRef);
+    dehttpize(zToFree);
+    zRef = zToFree;
+  }
   nBase = (int)strlen(g.zBaseURL);
-  if( fossil_strncmp(g.zBaseURL,zRef,nBase)!=0 ) return 0;
-  if( zRef[nBase]!=0 && zRef[nBase]!='/' ) return 0;
-  return 1;
+  if( fossil_strncmp(g.zBaseURL,zRef,nBase)!=0 ){
+    rc = 0;
+  }else if( zRef[nBase]!=0 && zRef[nBase]!='/' ){
+    rc = 0;
+  }else{
+    rc = 1;
+  }
+  if( rc==0 && bErrorLog && fossil_strcmp(P("REQUST_METHOD"),"POST")==0 ){
+    fossil_errorlog("warning: POST from different origin");
+  }
+  fossil_free(zToFree);
+  return rc;
 }
 
 /*
@@ -736,7 +764,7 @@ static int cgi_is_post_request(void){
 int cgi_csrf_safe(int securityLevel){
   if( g.okCsrf<0 ) return 0;
   if( g.okCsrf==0 ){
-    if( !cgi_same_origin() ){
+    if( !cgi_same_origin(1) ){
       g.okCsrf = -1;
     }else{
       g.okCsrf = 1;
@@ -924,6 +952,16 @@ void cgi_setenv(const char *zName, const char *zValue){
   cgi_set_parameter_nocopy(zName, fossil_strdup(zValue), 0);
 }
 
+/*
+** Returns true if NUL-terminated z contains any non-NUL
+** control characters (<0x20, 32d).
+*/
+static int contains_ctrl(const char *z){
+  assert(z);
+  for( ; *z>=0x20; ++z ){}
+  return 0!=*z;
+}
+
 /*
 ** Add a list of query parameters or cookies to the parameter set.
 **
@@ -943,7 +981,7 @@ void cgi_setenv(const char *zName, const char *zValue){
 **         environment variables always have uppercase names.
 **
 ** 2018-03-29:  Also ignore the entry if NAME that contains any characters
-** other than [a-zA-Z0-9_].  There are no known exploits involving unusual
+** other than [-a-zA-Z0-9_].  There are no known exploits involving unusual
 ** names that contain characters outside that set, but it never hurts to
 ** be extra cautious when sanitizing inputs.
 **
@@ -953,8 +991,12 @@ void cgi_setenv(const char *zName, const char *zValue){
 ** The input string "z" is modified but no copies is made.  "z"
 ** should not be deallocated or changed again after this routine
 ** returns or it will corrupt the parameter table.
+**
+** If bPermitCtrl is false and the decoded value of any entry in z
+** contains control characters (<0x20, 32d) then that key/value pair
+** are skipped.
 */
-static void add_param_list(char *z, int terminator){
+static void add_param_list(char *z, int terminator, int bPermitCtrl){
   int isQP = terminator=='&';
   while( *z ){
     char *zName;
@@ -977,7 +1019,10 @@ static void add_param_list(char *z, int terminator){
       zValue = "";
     }
     if( zName[0] && fossil_no_strange_characters(zName+1) ){
-      if( fossil_islower(zName[0]) ){
+      if( 0==bPermitCtrl && contains_ctrl(zValue) ){
+        continue /* Reject it. An argument could be made
+                 ** for break instead of continue. */;
+      }else if( fossil_islower(zName[0]) ){
         cgi_set_parameter_nocopy(zName, zValue, isQP);
       }else if( fossil_isupper(zName[0]) ){
         cgi_set_parameter_nocopy_tolower(zName, zValue, isQP);
@@ -1259,23 +1304,28 @@ void cgi_trace(const char *z){
 static NORETURN void malformed_request(const char *zMsg, ...);
 
 /*
-** Checks the QUERY_STRING environment variable, sets it up
-** via add_param_list() and, if found, applies its "skin"
-** setting. Returns 0 if no QUERY_STRING is set, 1 if it is,
-** and 2 if it sets the skin (in which case the cookie may
-** still need flushing by the page, via cookie_render()).
+** Checks the QUERY_STRING environment variable, sets it up via
+** add_param_list() and, if found, applies its "skin" setting. Returns
+** 0 if no QUERY_STRING is set, else it returns a bitmask of:
+**
+** 0x01 = QUERY_STRING was set up
+** 0x02 = "skin" URL param arg was processed
+** 0x04 = "x-f-l-c" cookie arg was processed.
+**
+*  In the case of the skin, the cookie may still need flushing
+** by the page, via cookie_render().
 */
 int cgi_setup_query_string(void){
   int rc = 0;
   char * z = (char*)P("QUERY_STRING");
   if( z ){
-    ++rc;
+    rc = 0x01;
     z = fossil_strdup(z);
-    add_param_list(z, '&');
+    add_param_list(z, '&', 0);
     z = (char*)P("skin");
     if( z ){
       char *zErr = skin_use_alternative(z, 2, SKIN_FROM_QPARAM);
-      ++rc;
+      rc |= 0x02;
       if( !zErr && P("once")==0 ){
         cookie_write_parameter("skin","skin",z);
         /* Per /chat discussion, passing ?skin=... without "once"
@@ -1286,6 +1336,14 @@ int cgi_setup_query_string(void){
       fossil_free(zErr);
     }
   }
+  if( !g.syncInfo.zLoginCard && 0!=(z=(char*)P("x-f-l-c")) ){
+    /* x-f-l-c (X-Fossil-Login-Card card transmitted via cookie
+    ** instead of in the sync payload. */
+    rc |= 0x04;
+    g.syncInfo.zLoginCard = fossil_strdup(z);
+    g.syncInfo.fLoginCardMode |= 0x02;
+    cgi_delete_parameter("x-f-l-c");
+  }
   return rc;
 }
 
@@ -1423,7 +1481,7 @@ void cgi_init(void){
   z = (char*)P("HTTP_COOKIE");
   if( z ){
     z = fossil_strdup(z);
-    add_param_list(z, ';');
+    add_param_list(z, ';', 0);
     z = (char*)cookie_value("skin",0);
     if(z){
       skin_use_alternative(z, 2, SKIN_FROM_COOKIE);
@@ -1486,7 +1544,7 @@ void cgi_decode_post_parameters(void){
     char *z = blob_str(&g.cgiIn);
     cgi_trace(z);
     if( g.zContentType[0]=='a' ){
-      add_param_list(z, '&');
+      add_param_list(z, '&', 1);
     }else{
       process_multipart_form_data(z, len);
     }
@@ -1579,6 +1637,21 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
   return zDefault;
 }
 
+/*
+** Return TRUE if the specific parameter exists and is a query parameter.
+** Return FALSE if the parameter is a cookie or environment variable.
+*/
+int cgi_is_qp(const char *zName){
+  int i;
+  if( zName==0 || fossil_isupper(zName[0]) ) return 0;
+  for(i=0; i<nUsedQP; i++){
+    if( fossil_strcmp(aParamQP[i].zName,zName)==0 ){
+      return aParamQP[i].isQP;
+    }
+  }
+  return 0;
+}
+
 /*
 ** Renders the "begone, spider" page and exits.
 */
@@ -1603,23 +1676,25 @@ static void cgi_begone_spider(const char *zName){
 }
 
 /*
-** If looks_like_sql_injection() returns true for the given string, calls
+** If looks_like_attack() returns true for the given string, call
 ** cgi_begone_spider() and does not return, else this function has no
 ** side effects. The range of checks performed by this function may
 ** be extended in the future.
 **
 ** Checks are omitted for any logged-in user.
 **
-** This is NOT a defense against SQL injection.  Fossil should easily be
-** proof against SQL injection without this routine.  Rather, this is an
-** attempt to avoid denial-of-service caused by persistent spiders that hammer
-** the server with dozens or hundreds of SQL injection attempts per second
-** against pages (such as /vdiff) that are expensive to compute.  In other
+** This is the primary defense against attack.  Fossil should easily be
+** proof against SQL injection and XSS attacks even without without this
+** routine.  Rather, this is an attempt to avoid denial-of-service caused
+** by persistent spiders that hammer the server with dozens or hundreds of
+** probes per seconds as they look for vulnerabilities. In other
 ** words, this is an effort to reduce the CPU load imposed by malicious
-** spiders.  It is not an effect defense against SQL injection vulnerabilities.
+** spiders.  Though those routine might help make attacks harder, it is
+** not itself an impenetrably barrier against attack and should not be
+** relied upon as the only defense.
 */
 void cgi_value_spider_check(const char *zTxt, const char *zName){
-  if( g.zLogin==0 && looks_like_sql_injection(zTxt) ){
+  if( g.zLogin==0 && looks_like_attack(zTxt) ){
     cgi_begone_spider(zName);
   }
 }
@@ -1629,7 +1704,7 @@ void cgi_value_spider_check(const char *zTxt, const char *zName){
 ** cgi_parameter(zName,zDefault) returns a value other than zDefault
 ** then it passes that value to cgi_value_spider_check().
 */
-const char *cgi_parameter_nosql(const char *zName, const char *zDefault){
+const char *cgi_parameter_no_attack(const char *zName, const char *zDefault){
   const char *zTxt = cgi_parameter(zName, zDefault);
 
   if( zTxt!=zDefault ){
@@ -1781,7 +1856,7 @@ void cgi_load_environment(void){
   static const char *const azCgiVars[] = {
     "COMSPEC", "DOCUMENT_ROOT", "GATEWAY_INTERFACE", "SCGI",
     "HTTP_ACCEPT", "HTTP_ACCEPT_CHARSET", "HTTP_ACCEPT_ENCODING",
-    "HTTP_ACCEPT_LANGUAGE", "HTTP_AUTHENICATION",
+    "HTTP_ACCEPT_LANGUAGE", "HTTP_AUTHENTICATION",
     "HTTP_CONNECTION", "HTTP_HOST",
     "HTTP_IF_NONE_MATCH", "HTTP_IF_MODIFIED_SINCE",
     "HTTP_USER_AGENT", "HTTP_REFERER", "PATH_INFO", "PATH_TRANSLATED",
@@ -1970,7 +2045,7 @@ static NORETURN void malformed_request(const char *zMsg, ...){
   z = vmprintf(zMsg, ap);
   va_end(ap);
   cgi_set_status(400, "Bad Request");
-  zContentType = "text/plain";
+  zReplyMimeType = "text/plain";
   if( g.zReqType==0 ) g.zReqType = "WWW";
   if( g.zReqType[0]=='C' && PD("SERVER_SOFTWARE",0)!=0 ){
     const char *zServer = PD("SERVER_SOFTWARE","");
@@ -2053,6 +2128,17 @@ static char *extract_token(char *zInput, char **zLeftOver){
   return zResult;
 }
 
+/*
+** All possible forms of an IP address.  Needed to work around GCC strict
+** aliasing rules.
+*/
+typedef union {
+  struct sockaddr sa;              /* Abstract superclass */
+  struct sockaddr_in sa4;          /* IPv4 */
+  struct sockaddr_in6 sa6;         /* IPv6 */
+  struct sockaddr_storage sas;     /* Should be the maximum of the above 3 */
+} address;
+
 /*
 ** Determine the IP address on the other side of a connection.
 ** Return a pointer to a string.  Or return 0 if unable.
@@ -2061,22 +2147,17 @@ static char *extract_token(char *zInput, char **zLeftOver){
 ** each call.
 */
 char *cgi_remote_ip(int fd){
-#if 0
-  static char zIp[100];
-  struct sockaddr_in6 addr;
-  socklen_t sz = sizeof(addr);
-  if( getpeername(fd, &addr, &sz) ) return 0;
-  zIp[0] = 0;
-  if( inet_ntop(AF_INET6, &addr, zIp, sizeof(zIp))==0 ){
+  address remoteAddr;
+  socklen_t size = sizeof(remoteAddr);
+  static char zHost[NI_MAXHOST];
+  if( getpeername(0, &remoteAddr.sa, &size) ){
     return 0;
   }
-  return zIp;
-#else
-  struct sockaddr_in remoteName;
-  socklen_t size = sizeof(struct sockaddr_in);
-  if( getpeername(fd, (struct sockaddr*)&remoteName, &size) ) return 0;
-  return inet_ntoa(remoteName.sin_addr);
-#endif
+  if( getnameinfo(&remoteAddr.sa, size, zHost, sizeof(zHost), 0, 0,
+                  NI_NUMERICHOST) ){
+    return 0;
+  }
+  return zHost;
 }
 
 /*
@@ -2096,6 +2177,7 @@ void cgi_handle_http_request(const char *zIpAddr){
   char zLine[2000];     /* A single line of input. */
   g.fullHttpReply = 1;
   g.zReqType = "HTTP";
+
   if( cgi_fgets(zLine, sizeof(zLine))==0 ){
     malformed_request("missing header");
   }
@@ -2109,7 +2191,7 @@ void cgi_handle_http_request(const char *zIpAddr){
    && fossil_strcmp(zToken,"POST")!=0
    && fossil_strcmp(zToken,"HEAD")!=0
   ){
-    malformed_request("unsupported HTTP method: \"%s\" - Fossil only supports"
+    malformed_request("unsupported HTTP method: \"%s\" - Fossil only supports "
                       "GET, POST, and HEAD", zToken);
   }
   cgi_setenv("GATEWAY_INTERFACE","CGI/1.0");
@@ -2132,7 +2214,6 @@ void cgi_handle_http_request(const char *zIpAddr){
     g.zIpAddr = fossil_strdup(zIpAddr);
   }
 
-
   /* Get all the optional fields that follow the first line.
   */
   while( cgi_fgets(zLine,sizeof(zLine)) ){
@@ -2469,7 +2550,6 @@ void cgi_handle_scgi_request(void){
   cgi_init();
 }
 
-
 #if INTERFACE
 /*
 ** Bitmap values for the flags parameter to cgi_http_server().
@@ -2512,118 +2592,218 @@ int cgi_http_server(
   /* Use win32_http_server() instead */
   fossil_exit(1);
 #else
-  int listener = -1;           /* The server socket */
-  int connection;              /* A socket for each individual connection */
+  int listen4 = -1;            /* Main socket; IPv4 or unix-domain */
+  int listen6 = -1;            /* Aux socket for corresponding IPv6 */
+  int mxListen = -1;           /* Maximum of listen4 and listen6 */
+  int connection;              /* An incoming connection */
   int nRequest = 0;            /* Number of requests handled so far */
   fd_set readfds;              /* Set of file descriptors for select() */
   socklen_t lenaddr;           /* Length of the inaddr structure */
   int child;                   /* PID of the child process */
   int nchildren = 0;           /* Number of child processes */
   struct timeval delay;        /* How long to wait inside select() */
-  struct sockaddr_in inaddr;   /* The socket address */
+  struct sockaddr_in6 inaddr6; /* Address for IPv6 */
+  struct sockaddr_in inaddr4;  /* Address for IPv4 */
   struct sockaddr_un uxaddr;   /* The address for unix-domain sockets */
   int opt = 1;                 /* setsockopt flag */
   int rc;                      /* Result code from system calls */
   int iPort = mnPort;          /* Port to try to use */
+  const char *zRequestType;    /* Type of requests to listen for */
 
-  while( iPort<=mxPort ){
-    if( flags & HTTP_SERVER_UNIXSOCKET ){
-      /* Initialize a Unix socket named g.zSockName */
-      assert( g.zSockName!=0 );
-      memset(&uxaddr, 0, sizeof(uxaddr));
-      if( strlen(g.zSockName)>sizeof(uxaddr.sun_path) ){
-        fossil_fatal("name of unix socket too big: %s\nmax size: %d\n",
-                     g.zSockName, (int)sizeof(uxaddr.sun_path));
-      }
-      if( file_isdir(g.zSockName, ExtFILE)!=0 ){
-        if( !file_issocket(g.zSockName) ){
-          fossil_fatal("cannot name socket \"%s\" because another object"
-                       " with that name already exists", g.zSockName);
-        }else{
-          unlink(g.zSockName);
-        }
-      }
-      uxaddr.sun_family = AF_UNIX;
-      strncpy(uxaddr.sun_path, g.zSockName, sizeof(uxaddr.sun_path)-1);
-      listener = socket(AF_UNIX, SOCK_STREAM, 0);
-      if( listener<0 ){
-        fossil_fatal("unable to create a unix socket named %s",
-                     g.zSockName);
-      }
-      /* Set the access permission for the new socket.  Default to 0660.
-      ** But use an alternative specified by --socket-mode if available.
-      ** Do this before bind() to avoid a race condition. */
-      if( g.zSockMode ){
-        file_set_mode(g.zSockName, listener, g.zSockMode, 0);
+
+  if( flags & HTTP_SERVER_SCGI ){
+    zRequestType = "SCGI";
+  }else if( g.httpUseSSL ){
+    zRequestType = "TLS-encrypted HTTPS";
+  }else{
+    zRequestType = "HTTP";
+  }
+
+  if( flags & HTTP_SERVER_UNIXSOCKET ){
+    /* CASE 1:  A unix socket named g.zSockName.  After creation, set the
+    **          permissions on the new socket to g.zSockMode and make the
+    **          owner of the socket be g.zSockOwner.
+    */
+    assert( g.zSockName!=0 );
+    memset(&uxaddr, 0, sizeof(uxaddr));
+    if( strlen(g.zSockName)>sizeof(uxaddr.sun_path) ){
+      fossil_fatal("name of unix socket too big: %s\nmax size: %d\n",
+                   g.zSockName, (int)sizeof(uxaddr.sun_path));
+    }
+    if( file_isdir(g.zSockName, ExtFILE)!=0 ){
+      if( !file_issocket(g.zSockName) ){
+        fossil_fatal("cannot name socket \"%s\" because another object"
+                     " with that name already exists", g.zSockName);
       }else{
-        file_set_mode(g.zSockName, listener, "0660", 1);
+        unlink(g.zSockName);
       }
+    }
+    uxaddr.sun_family = AF_UNIX;
+    strncpy(uxaddr.sun_path, g.zSockName, sizeof(uxaddr.sun_path)-1);
+    listen4 = socket(AF_UNIX, SOCK_STREAM, 0);
+    if( listen4<0 ){
+      fossil_fatal("unable to create a unix socket named %s",
+                   g.zSockName);
+    }
+    mxListen = listen4;
+    listen6 = -1;
+
+    /* Set the access permission for the new socket.  Default to 0660.
+    ** But use an alternative specified by --socket-mode if available.
+    ** Do this before bind() to avoid a race condition. */
+    if( g.zSockMode ){
+      file_set_mode(g.zSockName, listen4, g.zSockMode, 0);
     }else{
-      /* Initialize a TCP/IP socket on port iPort */
-      memset(&inaddr, 0, sizeof(inaddr));
-      inaddr.sin_family = AF_INET;
-      if( zIpAddr ){
-        inaddr.sin_addr.s_addr = inet_addr(zIpAddr);
-        if( inaddr.sin_addr.s_addr == INADDR_NONE ){
-          fossil_fatal("not a valid IP address: %s", zIpAddr);
-        }
-      }else if( flags & HTTP_SERVER_LOCALHOST ){
-        inaddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-      }else{
-        inaddr.sin_addr.s_addr = htonl(INADDR_ANY);
+      file_set_mode(g.zSockName, listen4, "0660", 1);
+    }
+    rc = bind(listen4, (struct sockaddr*)&uxaddr, sizeof(uxaddr));
+    /* Set the owner of the socket if requested by --socket-owner.  This
+    ** must wait until after bind(), after the filesystem object has been
+    ** created.  See https://lkml.org/lkml/2004/11/1/84 and
+    ** https://fossil-scm.org/forum/forumpost/7517680ef9684c57 */
+    if( g.zSockOwner ){
+      file_set_owner(g.zSockName, listen4, g.zSockOwner);
+    }
+    fossil_print("Listening for %s requests on unix socket %s\n",
+                 zRequestType, g.zSockName);
+    fflush(stdout);
+  }else if( zIpAddr && strchr(zIpAddr,':')!=0 ){
+    /* CASE 2: TCP on IPv6 IP address specified by zIpAddr and on port iPort.
+    */
+    assert( mnPort==mxPort );
+    memset(&inaddr6, 0, sizeof(inaddr6));
+    inaddr6.sin6_family = AF_INET6;
+    inaddr6.sin6_port = htons(iPort);
+    if( inet_pton(AF_INET6, zIpAddr, &inaddr6.sin6_addr)==0 ){
+      fossil_fatal("not a valid IPv6 address: %s", zIpAddr);
+    }
+    listen6 = socket(AF_INET6, SOCK_STREAM, 0);
+    if( listen6>0 ){
+      opt = 1;
+      setsockopt(listen6, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+      rc = bind(listen6, (struct sockaddr*)&inaddr6, sizeof(inaddr6));
+      if( rc<0 ){
+        close(listen6);
+        listen6 = -1;
       }
-      inaddr.sin_port = htons(iPort);
-      listener = socket(AF_INET, SOCK_STREAM, 0);
-      if( listener<0 ){
+    }
+    if( listen6<0 ){
+      fossil_fatal("cannot open a listening socket on [%s]:%d",
+                   zIpAddr, mnPort);
+    }
+    mxListen = listen6;
+    listen4 = -1;
+    fossil_print("Listening for %s requests on [%s]:%d\n",
+                 zRequestType, zIpAddr, iPort);
+    fflush(stdout);
+  }else if( zIpAddr && zIpAddr[0] ){
+    /* CASE 3: TCP on IPv4 IP address specified by zIpAddr and on port iPort.
+    */
+    assert( mnPort==mxPort );
+    memset(&inaddr4, 0, sizeof(inaddr4));
+    inaddr4.sin_family = AF_INET;
+    inaddr4.sin_port = htons(iPort);
+    if( strcmp(zIpAddr, "localhost")==0 ) zIpAddr = "127.0.0.1";
+    inaddr4.sin_addr.s_addr = inet_addr(zIpAddr);
+    if( inaddr4.sin_addr.s_addr == INADDR_NONE ){
+      fossil_fatal("not a valid IPv4 address: %s", zIpAddr);
+    }
+    listen4 = socket(AF_INET, SOCK_STREAM, 0);
+    if( listen4>0 ){
+      setsockopt(listen4, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+      rc = bind(listen4, (struct sockaddr*)&inaddr4, sizeof(inaddr4));
+      if( rc<0 ){
+        close(listen4);
+        listen4 = -1;
+      }
+    }
+    if( listen4<0 ){
+      fossil_fatal("cannot open a listening socket on %s:%d",
+                   zIpAddr, mnPort);
+    }
+    mxListen = listen4;
+    listen6 = -1;
+    fossil_print("Listening for %s requests on TCP port %s:%d\n",
+                 zRequestType, zIpAddr, iPort);
+    fflush(stdout);
+  }else{
+    /* CASE 4: Listen on all available IP addresses, or on only loopback
+    **         addresses (if HTTP_SERVER_LOCALHOST).  The TCP port is the
+    **         first available in the range of mnPort..mxPort.  Listen
+    **         on both IPv4 and IPv6, if possible.  The TCP port scan is done
+    **         on IPv4.
+    */
+    while( iPort<=mxPort ){
+      const char *zProto;
+      memset(&inaddr4, 0, sizeof(inaddr4));
+      inaddr4.sin_family = AF_INET;
+      inaddr4.sin_port = htons(iPort);
+      if( flags & HTTP_SERVER_LOCALHOST ){
+        inaddr4.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+      }else{
+        inaddr4.sin_addr.s_addr = htonl(INADDR_ANY);
+      }
+      listen4 = socket(AF_INET, SOCK_STREAM, 0);
+      if( listen4>0 ){
+        setsockopt(listen4, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+        rc = bind(listen4, (struct sockaddr*)&inaddr4, sizeof(inaddr4));
+        if( rc<0 ){
+          close(listen4);
+          listen4 = -1;
+        }
+      }
+      if( listen4<0 ){
         iPort++;
         continue;
       }
-    }
+      mxListen = listen4;
 
-    /* if we can't terminate nicely, at least allow the socket to be reused */
-    setsockopt(listener,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt));
-  
-    if( flags & HTTP_SERVER_UNIXSOCKET ){
-      rc = bind(listener, (struct sockaddr*)&uxaddr, sizeof(uxaddr));
-      /* Set the owner of the socket if requested by --socket-owner.  This
-      ** must wait until after bind(), after the filesystem object has been
-      ** created.  See https://lkml.org/lkml/2004/11/1/84 and
-      ** https://fossil-scm.org/forum/forumpost/7517680ef9684c57 */
-      if( g.zSockOwner ){
-        file_set_owner(g.zSockName, listener, g.zSockOwner);
+      /* If we get here, that means we found an open TCP port at iPort for
+      ** IPv4.  Try to set up a corresponding IPv6 socket on the same port.
+      */
+      memset(&inaddr6, 0, sizeof(inaddr6));
+      inaddr6.sin6_family = AF_INET6;
+      inaddr6.sin6_port = htons(iPort);
+      if( flags & HTTP_SERVER_LOCALHOST ){
+        inaddr6.sin6_addr = in6addr_loopback;
+      }else{
+        inaddr6.sin6_addr = in6addr_any;
       }
-    }else{
-      rc = bind(listener, (struct sockaddr*)&inaddr, sizeof(inaddr));
+      listen6 = socket(AF_INET6, SOCK_STREAM, 0);
+      if( listen6>0 ){
+        setsockopt(listen6, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+        setsockopt(listen6, IPPROTO_IPV6, IPV6_V6ONLY, &opt, sizeof(opt));
+        rc = bind(listen6, (struct sockaddr*)&inaddr6, sizeof(inaddr6));
+        if( rc<0 ){
+          close(listen6);
+          listen6 = -1;
+        }
+      }
+      if( listen6<0 ){
+        zProto = "IPv4 only";
+      }else{
+        zProto = "IPv4 and IPv6";
+        if( listen6>listen4 ) mxListen = listen6;
+      }
+
+      fossil_print("Listening for %s requests on TCP port %s%d, %s\n",
+                   zRequestType, 
+                   (flags & HTTP_SERVER_LOCALHOST)!=0 ? "localhost:" : "",
+                   iPort, zProto);
+      fflush(stdout);
+      break;
     }
-    if( rc<0 ){
-      close(listener);
-      iPort++;
-      continue;
-    }
-    break;
-  }
-  if( iPort>mxPort ){
-    if( flags & HTTP_SERVER_UNIXSOCKET ){
-      fossil_fatal("unable to listen on unix socket %s", zIpAddr);
-    }else if( mnPort==mxPort ){
-      fossil_fatal("unable to open listening socket on port %d", mnPort);
-    }else{
-      fossil_fatal("unable to open listening socket on any"
-                   " port in the range %d..%d", mnPort, mxPort);
+    if( iPort>mxPort ){
+      fossil_fatal("no available TCP ports in the range %d..%d",
+                   mnPort, mxPort);
     }
   }
-  if( iPort>mxPort ) return 1;
-  listen(listener,10);
-  if( flags & HTTP_SERVER_UNIXSOCKET ){
-    fossil_print("Listening for %s requests on unix socket %s\n",
-       (flags & HTTP_SERVER_SCGI)!=0 ? "SCGI" :
-          g.httpUseSSL?"TLS-encrypted HTTPS":"HTTP",  g.zSockName);
-  }else{
-    fossil_print("Listening for %s requests on TCP port %d\n",
-       (flags & HTTP_SERVER_SCGI)!=0 ? "SCGI" :
-          g.httpUseSSL?"TLS-encrypted HTTPS":"HTTP",  iPort);
-  }
-  fflush(stdout);
+
+  /* If we get to this point, that means there is at least one listening
+  ** socket on either listen4 or listen6 and perhaps on both. */
+  assert( listen4>0 || listen6>0 );
+  if( listen4>0 ) listen(listen4,10);
+  if( listen6>0 ) listen(listen6,10);
   if( zBrowser && (flags & HTTP_SERVER_UNIXSOCKET)==0 ){
     assert( strstr(zBrowser,"%d")!=0 );
     zBrowser = mprintf(zBrowser /*works-like:"%d"*/, iPort);
@@ -2641,6 +2821,11 @@ int cgi_http_server(
       fossil_warning("cannot start browser: %s\n", zBrowser);
     }
   }
+
+  /* What for incomming requests.  For each request, fork() a child process
+  ** to deal with that request.  The child process returns.  The parent
+  ** keeps on listening and never returns.
+  */
   while( 1 ){
 #if FOSSIL_MAX_CONNECTIONS>0
     while( nchildren>=FOSSIL_MAX_CONNECTIONS ){
@@ -2650,43 +2835,51 @@ int cgi_http_server(
     delay.tv_sec = 0;
     delay.tv_usec = 100000;
     FD_ZERO(&readfds);
-    assert( listener>=0 );
-    FD_SET( listener, &readfds);
-    select( listener+1, &readfds, 0, 0, &delay);
-    if( FD_ISSET(listener, &readfds) ){
-      lenaddr = sizeof(inaddr);
-      connection = accept(listener, (struct sockaddr*)&inaddr, &lenaddr);
-      if( connection>=0 ){
-        if( flags & HTTP_SERVER_NOFORK ){
-          child = 0;
-        }else{
-          child = fork();
+    assert( listen4>0 || listen6>0 );
+    if( listen4>0 ) FD_SET( listen4, &readfds);
+    if( listen6>0 ) FD_SET( listen6, &readfds);
+    select( mxListen+1, &readfds, 0, 0, &delay);
+    if( listen4>0 && FD_ISSET(listen4, &readfds) ){
+      lenaddr = sizeof(inaddr4);
+      connection = accept(listen4, (struct sockaddr*)&inaddr4, &lenaddr);
+    }else if( listen6>0 && FD_ISSET(listen6, &readfds) ){
+      lenaddr = sizeof(inaddr6);
+      connection = accept(listen6, (struct sockaddr*)&inaddr6, &lenaddr);
+    }else{
+      connection = -1;
+    }
+    if( connection>=0 ){
+      if( flags & HTTP_SERVER_NOFORK ){
+        child = 0;
+      }else{
+        child = fork();
+      }
+      if( child!=0 ){
+        if( child>0 ){
+          nchildren++;
+          nRequest++;
         }
-        if( child!=0 ){
-          if( child>0 ){
-            nchildren++;
-            nRequest++;
-          }
-          close(connection);
-        }else{
-          int nErr = 0, fd;
-          g.zSockName = 0 /* avoid deleting the socket via atexit() */;
-          close(0);
+        close(connection);
+      }else{
+        int nErr = 0, fd;
+        g.zSockName = 0 /* avoid deleting the socket via atexit() */;
+        close(0);
+        fd = dup(connection);
+        if( fd!=0 ) nErr++;
+        close(1);
+        fd = dup(connection);
+        if( fd!=1 ) nErr++;
+        if( 0 && !g.fAnyTrace ){
+          close(2);
           fd = dup(connection);
-          if( fd!=0 ) nErr++;
-          close(1);
-          fd = dup(connection);
-          if( fd!=1 ) nErr++;
-          if( 0 && !g.fAnyTrace ){
-            close(2);
-            fd = dup(connection);
-            if( fd!=2 ) nErr++;
-          }
-          close(connection);
-          g.nPendingRequest = nchildren+1;
-          g.nRequest = nRequest+1;
-          return nErr;
+          if( fd!=2 ) nErr++;
         }
+        close(connection);
+        if( listen4>0 ) close(listen4);
+        if( listen6>0 ) close(listen6);
+        g.nPendingRequest = nchildren+1;
+        g.nRequest = nRequest+1;
+        return nErr;
       }
     }
     /* Bury dead children */
