@@ -257,8 +257,9 @@ void chat_webpage(void){
          "  fromcli: %h,\n"
          "  alertSound: \"%h\",\n"
          "  initSize: %d,\n"
-         "  imagesInline: !!%d\n"
-         "};\n",(PB("cli")?"true":"false"),(zAlert),(db_get_int("chat-initial-history",50)),(db_get_boolean("chat-inline-images",1)));
+         "  imagesInline: !!%d,\n"
+         "  pollTimeout: %d\n"
+         "};\n",(PB("cli")?"true":"false"),(zAlert),(db_get_int("chat-initial-history",50)),(db_get_boolean("chat-inline-images",1)),(db_get_int("chat-poll-timeout",420)));
   ajax_emit_js_preview_modes(0);
   chat_emit_alert_list();
   cgi_printf("}, false);\n"
@@ -392,6 +393,21 @@ static void chat_emit_permissions_error(int fAsMessageList){
   fossil_free(zTime);
 }
 
+/*
+** Like chat_emit_permissions_error() but emits a single
+** /chat-message-format JSON object about a CSRF violation.
+*/
+static void chat_emit_csrf_error(void){
+  char * zTime = cgi_iso8601_datestamp();
+  cgi_set_content_type("application/json");
+  CX("{");
+  CX("\"isError\": true, \"xfrom\": null,");
+  CX("\"mtime\": %!j, \"lmtime\": %!j,", zTime, zTime);
+  CX("\"xmsg\": \"CSRF validation failure.\"");
+  CX("}");
+  fossil_free(zTime);
+}
+
 /*
 ** WEBPAGE: chat-send hidden loadavg-exempt
 **
@@ -424,6 +440,9 @@ void chat_send_webpage(void){
   if( 0==g.perm.Chat ) {
     chat_emit_permissions_error(0);
     return;
+  }else if( 0==cgi_csrf_safe(1) ){
+    chat_emit_csrf_error();
+    return;
   }
   zUserName = (g.zLogin && g.zLogin[0]) ? g.zLogin : "nobody";
   nByte = atoi(PD("file:bytes","0"));
@@ -1205,7 +1224,7 @@ void chat_msg_from_event(
 **                               previously undownloaded content is retrieved.
 **        --debug                Additional debugging output
 **        --out DATABASE         Store CHAT table in separate database file
-**                               DATABASE rather that adding to local clone
+**                               DATABASE rather than adding to local clone
 **        --unsafe               Allow the use of unencrypted http://
 **
 ** > fossil chat send [ARGUMENTS]
@@ -1323,7 +1342,7 @@ void chat_command(void){
                        "\r\n%s\r\n%s", zMsg, zBoundary);
     }
     if( zFilename && blob_read_from_file(&fcontent, zFilename, ExtFILE)>0 ){
-      char *zFN = mprintf("%s", file_tail(zAs ? zAs : zFilename));
+      char *zFN = fossil_strdup(file_tail(zAs ? zAs : zFilename));
       int i;
       const char *zMime = mimetype_from_name(zFN);
       for(i=0; zFN[i]; i++){
