@@ -603,6 +603,7 @@ int content_put_ex(
     db_exec(&s1);
     rid = db_last_insert_rowid();
     if( !pBlob ){
+      assert(!"cannot happen: pBlob is always non-NULL");
       db_multi_exec("INSERT OR IGNORE INTO phantom VALUES(%d)", rid);
     }
   }
@@ -949,6 +950,7 @@ static int looks_like_control_artifact(Blob *p){
   int n = blob_size(p);
   if( n<10 ) return 0;
   if( strncmp(z, "-----BEGIN PGP SIGNED MESSAGE-----", 34)==0 ) return 1;
+  if( strncmp(z, "-----BEGIN SSH SIGNED MESSAGE-----", 34)==0 ) return 1;
   if( z[0]<'A' || z[0]>'Z' || z[1]!=' ' || z[0]=='I' ) return 0;
   if( z[n-1]!='\n' ) return 0;
   return 1;
