@@ -89,7 +89,7 @@ void cookie_parse(void){
   if( cookies.bIsInit ) return;
   z = (char*)P(DISPLAY_SETTINGS_COOKIE);
   if( z==0 ) z = "";
-  cookies.zCookieValue = z = mprintf("%s", z);
+  cookies.zCookieValue = z = fossil_strdup(z);
   cookies.bIsInit = 1;
   while( cookies.nParam<COOKIE_NPARAM ){
     while( fossil_isspace(z[0]) ) z++;
@@ -259,7 +259,9 @@ void cookie_page(void){
     if( bFDSonly && strcmp(zName, "fossil_display_settings")!=0 ) continue;
     zDel = mprintf("del%s",zName);
     if( P(zDel)!=0 ){
-      cgi_set_cookie(zName, "", 0, -1);
+      const char *zPath = fossil_strcmp(ROBOT_COOKIE,zName)==0
+        ? "/" : 0;
+      cgi_set_cookie(zName, "", zPath, -1);
       cgi_redirect(g.zPath);
     }
     nCookie++;
@@ -285,6 +287,10 @@ void cookie_page(void){
     ){
       cgi_printf("<p>This appears to be a login cookie for another Fossil repository\n"
              "in the same website.\n");
+    }else
+    if( fossil_strcmp(zName, ROBOT_COOKIE)==0 ){
+      cgi_printf("<p>This cookie shows that your web-browser has been tested is\n"
+             "believed to be operated by a human, not a robot.\n");
     }
     else {
       cgi_printf("<p>This cookie was not generated by Fossil.  It might be something\n"
