@@ -232,7 +232,7 @@ void event_page(void){
                        "  FROM tag"
                        " WHERE tagname GLOB 'event-%q*'",
                     zId);
-  attachment_list(zFullId, "<hr><h2>Attachments:</h2><ul>");
+  attachment_list(zFullId, "<h2>Attachments:</h2>", 1);
   document_emit_js();
   style_finish_page();
   manifest_destroy(pTNote);
@@ -385,7 +385,7 @@ void eventedit_page(void){
   int isNew = 0;
 
   if( zBody ){
-    zBody = mprintf("%s", zBody);
+    zBody = fossil_strdup(zBody);
   }
   login_check_credentials();
   zId = P("name");
