@@ -61,7 +61,7 @@ static void print_person(const char *zUser){
   db_bind_text(&q, ":user", zUser);
   if( db_step(&q)!=SQLITE_ROW ){
     db_reset(&q);
-    zName = mprintf("%s", zUser);
+    zName = fossil_strdup(zUser);
     for(i=j=0; zName[i]; i++){
       if( zName[i]!='<' && zName[i]!='>' && zName[i]!='"' ){
         zName[j++] = zName[i];
@@ -105,7 +105,7 @@ static void print_person(const char *zUser){
   }
   if( zContact[i]==0 ){
     /* No email address found. Take as user info if not empty */
-    zName = mprintf("%s", zContact[0] ? zContact : zUser);
+    zName = fossil_strdup(zContact[0] ? zContact : zUser);
     for(i=j=0; zName[i]; i++){
       if( zName[i]!='<' && zName[i]!='>' && zName[i]!='"' ){
         zName[j++] = zName[i];
@@ -152,7 +152,7 @@ static void print_person(const char *zUser){
      }
   }
 
-  if( zName==NULL ) zName = mprintf("%s", zUser);
+  if( zName==NULL ) zName = fossil_strdup(zUser);
   for(i=j=0; zName[i]; i++){
      if( zName[i]!='<' && zName[i]!='>' && zName[i]!='"' ){
          zName[j++] = zName[i];
@@ -175,7 +175,7 @@ static void print_person(const char *zUser){
 ** the branch or tag part of the reference.
 */
 static void print_ref(const char *zRef){
-  char *zEncoded = mprintf("%s", zRef);
+  char *zEncoded = fossil_strdup(zRef);
   int i, w;
   if (zEncoded[0]=='@' && zEncoded[1]=='\0'){
     putchar(REFREPLACEMENT);
@@ -484,6 +484,8 @@ void export_marks(FILE* f, Bag *blobs, Bag *vers){
 /*
 ** COMMAND: export*
 **
+** Usage: %fossil export --git [REPOSITORY]
+**
 ** This command is deprecated.  Use "fossil git export" instead.
 */
 void export_cmd(void){
@@ -1075,8 +1077,7 @@ static int gitmirror_send_checkin(
   FILE *xCmd,           /* Write fast-import text on this pipe */
   int rid,              /* BLOB.RID for the check-in to export */
   const char *zUuid,    /* BLOB.UUID for the check-in to export */
-  int *pnLimit,         /* Stop when the counter reaches zero */
-  int fManifest         /* MFESTFLG_* values */
+  int *pnLimit          /* Stop when the counter reaches zero */
 ){
   Manifest *pMan;       /* The check-in to be output */
   int i;                /* Loop counter */
@@ -1090,6 +1091,8 @@ static int gitmirror_send_checkin(
   int bPhantomOk;       /* True if phantom files should be ignored */
   char buf[24];
   char *zEmail;         /* Contact info for Git committer field */
+  int fManifest;        /* Should the manifest files be included? */
+  int fPManifest = 0;   /* OR of the manifest files for all parents */
 
   pMan = manifest_get(rid, CFTYPE_MANIFEST, 0);
   if( pMan==0 ){
@@ -1107,7 +1110,7 @@ static int gitmirror_send_checkin(
       int prid = db_int(0, "SELECT rid FROM blob WHERE uuid=%Q",
                         pMan->azParent[i]);
       int rc = gitmirror_send_checkin(xCmd, prid, pMan->azParent[i],
-                                      pnLimit, fManifest);
+                                      pnLimit);
       if( rc || *pnLimit<=0 ){
         manifest_destroy(pMan);
         return 1;
@@ -1152,7 +1155,7 @@ static int gitmirror_send_checkin(
   if( fossil_strcmp(zBranch,"trunk")==0 ){
     assert( gitmirror_mainbranch!=0 );
     fossil_free(zBranch);
-    zBranch = mprintf("%s",gitmirror_mainbranch);
+    zBranch = fossil_strdup(gitmirror_mainbranch);
   }else if( zBranch==0 ){
     zBranch = mprintf("unknown");
   }else{
@@ -1216,6 +1219,7 @@ static int gitmirror_send_checkin(
   for(i=0; i<pMan->nParent; i++){
     char *zOther = gitmirror_find_mark(pMan->azParent[i],0,0);
     if( zOther==0 ) continue;
+    fPManifest |= db_get_manifest_setting(pMan->azParent[i]);
     if( iParent<0 ){
       iParent = i;
       fprintf(xCmd, "from %s\n", zOther);
@@ -1272,6 +1276,7 @@ static int gitmirror_send_checkin(
   pMan = 0;
 
   /* Include Fossil-generated auxiliary files in the check-in */
+  fManifest = db_get_manifest_setting(zUuid);
   if( fManifest & MFESTFLG_RAW ){
     Blob manifest;
     content_get(rid, &manifest);
@@ -1279,10 +1284,14 @@ static int gitmirror_send_checkin(
     fprintf(xCmd,"M 100644 inline manifest\ndata %d\n%s\n",
       blob_strlen(&manifest), blob_str(&manifest));
     blob_reset(&manifest);
+  }else if( fPManifest & MFESTFLG_RAW ){
+    fprintf(xCmd, "D manifest\n");
   }
   if( fManifest & MFESTFLG_UUID ){
     int n = (int)strlen(zUuid);
     fprintf(xCmd,"M 100644 inline manifest.uuid\ndata %d\n%s\n\n", n+1, zUuid);
+  }else if( fPManifest & MFESTFLG_UUID ){
+    fprintf(xCmd, "D manifest.uuid\n");
   }
   if( fManifest & MFESTFLG_TAGS ){
     Blob tagslist;
@@ -1291,6 +1300,8 @@ static int gitmirror_send_checkin(
     fprintf(xCmd,"M 100644 inline manifest.tags\ndata %d\n%s\n",
       blob_strlen(&tagslist), blob_str(&tagslist));
     blob_reset(&tagslist);
+  }else if( fPManifest & MFESTFLG_TAGS ){
+    fprintf(xCmd, "D manifest.tags\n");
   }
 
   /* The check-in is finished, so decrement the counter */
@@ -1384,7 +1395,6 @@ void gitmirror_export_command(void){
   int rc;                         /* Result code */
   int bForce;                     /* Do the export and sync even if no changes*/
   int bNeedRepack = 0;            /* True if we should run repack at the end */
-  int fManifest;                  /* Current "manifest" setting */
   int bIfExists;                  /* The --if-mirrored flag */
   FILE *xCmd;                     /* Pipe to the "git fast-import" command */
   FILE *pMarks;                   /* Git mark files */
@@ -1522,9 +1532,6 @@ void gitmirror_export_command(void){
     return;
   }
 
-  /* Do we need to include manifest files in the clone? */
-  fManifest = db_get_manifest_setting();
-
   /* Change to the MIRROR directory so that the Git commands will work */
   rc = file_chdir(zMirror, 0);
   if( rc ) fossil_fatal("cannot change the working directory to \"%s\"",
@@ -1580,7 +1587,7 @@ void gitmirror_export_command(void){
     double rMTime = db_column_double(&q, 1);
     const char *zUuid = db_column_text(&q, 2);
     if( rMTime>rEnd ) rEnd = rMTime;
-    rc = gitmirror_send_checkin(xCmd, rid, zUuid, &nLimit, fManifest);
+    rc = gitmirror_send_checkin(xCmd, rid, zUuid, &nLimit);
     if( rc ) break;
     gitmirror_message(VERB_NORMAL,"%d/%d      \r", nTotal-nLimit, nTotal);
     fflush(stdout);
