@@ -40,7 +40,7 @@
 ** option to see similar information about the same file for the check-in
 ** specified by VERSION.
 **
-** In the -s mode prints the status as <status> <revision>.  This is
+** The -s mode prints the status as <status> <revision>.  This is
 ** a quick status and does not check for up-to-date-ness of the file.
 **
 ** In the -p mode, there's an optional flag "-r|--revision REVISION".
@@ -501,12 +501,12 @@ void finfo_page(void){
   );
   if( (zA = P("a"))!=0 ){
     blob_append_sql(&sql, "  AND event.mtime>=%.16g\n",
-         symbolic_name_to_mtime(zA,0));
+         symbolic_name_to_mtime(zA,0,0));
     url_add_parameter(&url, "a", zA);
   }
   if( (zB = P("b"))!=0 ){
     blob_append_sql(&sql, "  AND event.mtime<=%.16g\n",
-         symbolic_name_to_mtime(zB,0));
+         symbolic_name_to_mtime(zB,0,1));
     url_add_parameter(&url, "b", zB);
   }
   if( ridFrom ){
@@ -639,6 +639,8 @@ void finfo_page(void){
       zBgClr = user_color(zUser);
     }else if( brBg || zBgClr==0 || zBgClr[0]==0 ){
       zBgClr = strcmp(zBr,"trunk")==0 ? "" : hash_color(zBr);
+    }else if( zBgClr ){
+      zBgClr = reasonable_bg_color(zBgClr,0);
     }
     gidx = graph_add_row(pGraph,
                    frid>0 ? (GraphRowId)frid*(mxfnid+1)+fnid : fpid+1000000000,
