@@ -62,6 +62,7 @@ struct ForumThread {
   ForumPost *pDisplay;   /* Entries in display order */
   ForumPost *pTail;      /* Last on the display list */
   int mxIndent;          /* Maximum indentation level */
+  int nArtifact;         /* Number of forum artifacts in this thread */
 };
 #endif /* INTERFACE */
 
@@ -112,8 +113,13 @@ static int forumpost_head_rid(int rid){
 ** If bCheckIrt is true then p's thread in-response-to parents are
 ** checked (recursively) for closure, else only p is checked.
 */
-static int forumpost_is_closed(ForumPost *p, int bCheckIrt){
-  while(p){
+static int forumpost_is_closed(
+  ForumThread *pThread,          /* Thread that the post is a member of */
+  ForumPost *p,                  /* the forum post */
+  int bCheckIrt                  /* True to check In-Reply-To posts */
+){
+  int mx = pThread->nArtifact+1;
+  while( p && (mx--)>0 ){
     if( p->pEditHead ) p = p->pEditHead;
     if( p->iClosed || !bCheckIrt ) return p->iClosed;
     p = p->pIrt;
@@ -412,6 +418,7 @@ static ForumThread *forumthread_create(int froot, int computeHierarchy){
       pThread->pLast->pNext = pPost;
     }
     pThread->pLast = pPost;
+    pThread->nArtifact++;
 
     /* Find the in-reply-to post.  Default to the topic post if the replied-to
     ** post cannot be found. */
@@ -523,6 +530,7 @@ void forumthread_cmd(void){
   fossil_print("fpid  = %d\n", fpid);
   fossil_print("froot = %d\n", froot);
   pThread = forumthread_create(froot, 1);
+  fossil_print("count = %d\n", pThread->nArtifact);
   fossil_print("Chronological:\n");
   fossil_print(
 /* 0         1         2         3         4         5         6         7    */
@@ -568,6 +576,7 @@ void forumthreadhashlist(void){
   ForumThread *pThread;
   ForumPost *p;
   char *fuuid;
+  Stmt q;
 
   login_check_credentials();
   if( !g.perm.Admin ){
@@ -601,7 +610,24 @@ void forumthreadhashlist(void){
     cgi_printf("%h\n",(p->zUuid));
   }
   forumthread_delete(pThread);
-  cgi_printf("</pre>\n");
+  cgi_printf("</pre>\n"
+         "<hr>\n"
+         "<h2>Related FORUMPOST Table Content</h2>\n"
+         "<table border=\"1\" cellpadding=\"4\" cellspacing=\"0\">\n"
+         "<tr><th>fpid<th>froot<th>fprev<th>firt<th>fmtime\n");
+  db_prepare(&q, "SELECT fpid, froot, fprev, firt, datetime(fmtime)"
+                 "  FROM forumpost"
+                 " WHERE froot=%d"
+                 " ORDER BY fmtime", froot);
+  while( db_step(&q)==SQLITE_ROW ){
+    cgi_printf("<tr><td>%d"
+           "<td>%d"
+           "<td>%d"
+           "<td>%d"
+           "<td>%h</tr>\n",(db_column_int(&q,0)),(db_column_int(&q,1)),(db_column_int(&q,2)),(db_column_int(&q,3)),(db_column_text(&q,4)));
+  }
+  cgi_printf("</table>\n");
+  db_finalize(&q);
   style_finish_page();
 }
 
@@ -728,6 +754,7 @@ static char *forum_post_display_name(ForumPost *p, Manifest *pManifest){
 ** Display a single post in a forum thread.
 */
 static void forum_display_post(
+  ForumThread *pThread, /* The thread that this post is a member of */
   ForumPost *p,         /* Forum post to display */
   int iIndentScale,     /* Indent scale factor */
   int bRaw,             /* True to omit the border */
@@ -750,10 +777,10 @@ static void forum_display_post(
   /* Get the manifest for the post.  Abort if not found (e.g. shunned). */
   pManifest = manifest_get(p->fpid, CFTYPE_FORUM, 0);
   if( !pManifest ) return;
-  iClosed = forumpost_is_closed(p, 1);
+  iClosed = forumpost_is_closed(pThread, p, 1);
   /* When not in raw mode, create the border around the post. */
   if( !bRaw ){
-    /* Open the <div> enclosing the post.  Set the class string to mark the post
+    /* Open the <div> enclosing the post. Set the class string to mark the post
     ** as selected and/or obsolete. */
     iIndent = (p->pEditHead ? p->pEditHead->nIndent : p->nIndent)-1;
     cgi_printf("<div id='forum%d' class='forumTime"
@@ -909,7 +936,8 @@ static void forum_display_post(
         login_insert_csrf_secret();
         cgi_printf("<input type=\"hidden\" name=\"fpid\" value=\"%z\" />\n",(rid_to_uuid(iHead)));
         if( moderation_pending(p->fpid)==0 ){
-          cgi_printf("<input type=\"submit\" value='%s' />\n",(iClosed ? "Re-open" : "Close"));
+          cgi_printf("<input type=\"button\" value='%s' "
+                 " class='%s'/>\n",(iClosed ? "Re-open" : "Close"),(iClosed ? "action-reopen" : "action-close"));
         }
         cgi_printf("</form>\n");
       }
@@ -1030,7 +1058,7 @@ static void forum_display_thread(
   /* Display the appropriate subset of posts in sequence. */
   while( p ){
     /* Display the post. */
-    forum_display_post(p, iIndentScale, mode==FD_RAW,
+    forum_display_post(pThread, p, iIndentScale, mode==FD_RAW,
         bUnf, bHist, p==pSelect, zQuery);
 
     /* Advance to the next post in the thread. */
@@ -1090,7 +1118,8 @@ static void forum_display_thread(
 ** code (e.g. "forum.js").
 */
 static void forum_emit_js(void){
-  builtin_fossil_js_bundle_or("copybutton", "pikchr", NULL);
+  builtin_fossil_js_bundle_or("copybutton", "pikchr", "confirmer",
+                              NULL);
   builtin_request_js("fossil.page.forumpost.js");
 }
 
@@ -1764,6 +1793,21 @@ void forumedit_page(void){
   style_finish_page();
 }
 
+/*
+** SETTING: forum-close-policy    boolean default=off
+** If true, forum moderators may close/re-open forum posts, and reply
+** to closed posts. If false, only administrators may do so. Note that
+** this only affects the forum web UI, not post-closing tags which
+** arrive via the command-line or from synchronization with a remote.
+*/
+/*
+** SETTING: forum-title          width=20 default=Forum
+** This is the name or "title" of the Forum for this repository.  The
+** default is just "Forum".  But in some setups, admins might want to
+** change it to "Developer Forum" or "User Forum" or whatever other name
+** seems more appropriate for the particular usage.
+*/
+
 /*
 ** WEBPAGE: setup_forum
 **
@@ -1771,9 +1815,9 @@ void forumedit_page(void){
 */
 void forum_setup(void){
   /* boolean config settings specific to the forum. */
-  const char * zSettingsBool[] = {
-  "forum-close-policy",
-  NULL /* sentinel entry */
+  static const char *azForumSettings[] =  {
+    "forum-close-policy",
+    "forum-title",
   };
 
   login_check_credentials();
@@ -1791,11 +1835,9 @@ void forum_setup(void){
            "<a href='%R/timeline?y=f'>%d</a></p>\n",(nPosts));
   }
 
-  cgi_printf("<h2>Supervisors</h2>\n"
-         "<p>Users with capabilities 's', 'a', or '6'.</p>\n");
+  cgi_printf("<h2>Supervisors</h2>\n");
   {
     Stmt q = empty_Stmt;
-    int nRows = 0;
     db_prepare(&q, "SELECT uid, login, cap FROM user "
                    "WHERE cap GLOB '*[as6]*' ORDER BY login");
     cgi_printf("<table class='bordered'>\n"
@@ -1805,7 +1847,6 @@ void forum_setup(void){
       const int iUid = db_column_int(&q, 0);
       const char *zUser = db_column_text(&q, 1);
       const char *zCap = db_column_text(&q, 2);
-      ++nRows;
       cgi_printf("<tr>\n"
              "<td><a href='%R/setup_uedit?id=%d'>%h</a></td>\n"
              "<td>(%h)</td>\n"
@@ -1813,20 +1854,17 @@ void forum_setup(void){
     }
     db_finalize(&q);
    cgi_printf("</tbody></table>\n");
-    if( 0==nRows ){
-      cgi_printf("No supervisors\n");
-    }else{
-      cgi_printf("%d supervisor(s)\n",(nRows));
-    }
   }
 
-  cgi_printf("<h2>Moderators</h2>\n"
-         "<p>Users with capability '5'.</p>\n");
-  {
+  cgi_printf("<h2>Moderators</h2>\n");
+  if( db_int(0, "SELECT count(*) FROM user "
+                " WHERE cap GLOB '*5*' AND cap NOT GLOB '*[as6]*'")==0 ){
+      cgi_printf("<p>No non-supervisor moderators\n");
+  }else{
     Stmt q = empty_Stmt;
-    int nRows = 0;
     db_prepare(&q, "SELECT uid, login, cap FROM user "
-               "WHERE cap GLOB '*5*' ORDER BY login");
+               "WHERE cap GLOB '*5*' AND cap NOT GLOB '*[as6]*'"
+               " ORDER BY login");
     cgi_printf("<table class='bordered'>\n"
            "<thead><tr><th>User</th><th>Capabilities</th></tr></thead>\n"
            "<tbody>\n");
@@ -1834,7 +1872,6 @@ void forum_setup(void){
       const int iUid = db_column_int(&q, 0);
       const char *zUser = db_column_text(&q, 1);
       const char *zCap = db_column_text(&q, 2);
-      ++nRows;
       cgi_printf("<tr>\n"
              "<td><a href='%R/setup_uedit?id=%d'>%h</a></td>\n"
              "<td>(%h)</td>\n"
@@ -1842,39 +1879,55 @@ void forum_setup(void){
     }
     db_finalize(&q);
     cgi_printf("</tbody></table>\n");
-    if( 0==nRows ){
-      cgi_printf("No non-supervisor moderators\n");
-    }else{
-      cgi_printf("%d moderator(s)\n",(nRows));
-    }
   }
 
-  cgi_printf("<h2>Settings</h2>\n"
-         "<p>Configuration settings specific to the forum.</p>\n");
+  cgi_printf("<h2>Settings</h2>\n");
   if( P("submit") && cgi_csrf_safe(2) ){
     int i = 0;
-    const char *zSetting;
     db_begin_transaction();
-    while( (zSetting = zSettingsBool[i++]) ){
-      const char *z = P(zSetting);
-      if( !z || !z[0] ) z = "off";
-      db_set(zSetting/*works-like:"x"*/, z, 0);
+    for(i=0; i<ArraySize(azForumSettings); i++){
+      char zQP[4];
+      const char *z;
+      const Setting *pSetting = setting_find(azForumSettings[i]);
+      if( pSetting==0 ) continue;
+      zQP[0] = 'a'+i;
+      zQP[1] = zQP[0];
+      zQP[2] = 0;
+      z = P(zQP);
+      if( z==0 || z[0]==0 ) continue;
+      db_set(pSetting->name/*works-like:"x"*/, z, 0);
     }
     db_end_transaction(0);
     cgi_printf("<p><em>Settings saved.</em></p>\n");
   }
   {
     int i = 0;
-    const char *zSetting;
     cgi_printf("<form action=\"%R/setup_forum\" method=\"post\">\n");
     login_insert_csrf_secret();
     cgi_printf("<table class='forum-settings-list'><tbody>\n");
-    while( (zSetting = zSettingsBool[i++]) ){
-      cgi_printf("<tr><td>\n");
-      onoff_attribute("", zSetting, zSetting/*works-like:"x"*/, 0, 0);
-      cgi_printf("</td><td>\n"
-             "<a href='%R/help?cmd=%h'>%h</a>\n"
-             "</td></tr>\n",(zSetting),(zSetting));
+    for(i=0; i<ArraySize(azForumSettings); i++){
+      char zQP[4];
+      const Setting *pSetting = setting_find(azForumSettings[i]);
+      if( pSetting==0 ) continue;
+      zQP[0] = 'a'+i;
+      zQP[1] = zQP[0];
+      zQP[2] = 0;
+      if( pSetting->width==0 ){
+        /* Boolean setting */
+        cgi_printf("<tr><td align=\"right\">\n"
+               "<a href='%R/help?cmd=%h'>%h</a>:\n"
+               "</td><td>\n",(pSetting->name),(pSetting->name));
+        onoff_attribute("", zQP, pSetting->name/*works-like:"x"*/, 0, 0);
+        cgi_printf("</td></tr>\n");
+      }else{
+        /* Text value setting */
+        cgi_printf("<tr><td align=\"right\">\n"
+               "<a href='%R/help?cmd=%h'>%h</a>:\n"
+               "</td><td>\n",(pSetting->name),(pSetting->name));
+        entry_attribute("", 25, pSetting->name, zQP/*works-like:""*/,
+                        pSetting->def, 0);
+        cgi_printf("</td></tr>\n");
+      }   
     }
     cgi_printf("</tbody></table>\n"
            "<input type='submit' name='submit' value='Apply changes'>\n"
@@ -1913,7 +1966,8 @@ void forum_main_page(void){
   }
   cgi_check_for_malice();
   style_set_current_feature("forum");
-  style_header( "%s", isSearch ? "Forum Search Results" : "Forum" );
+  style_header("%s%s", db_get("forum-title","Forum"), 
+                       isSearch ? " Search Results" : "");
   style_submenu_element("Timeline", "%R/timeline?ss=v&y=f&vfx");
   if( g.perm.WrForum ){
     style_submenu_element("New Thread","%R/forumnew");
