@@ -47,6 +47,26 @@ void loadavg_test_cmd(void){
   fossil_print("load-average: %f\n", load_average());
 }
 
+/*
+** WEBPAGE:  test-overload
+**
+** Generate the response that would normally be shown only when
+** service is denied due to an overload condition.  This is for
+** testing of the overload warning page.
+*/
+void overload_page(void){
+  double mxLoad = atof(db_get("max-loadavg", "0.0"));
+  style_set_current_feature("test");
+  style_header("Server Overload");
+  cgi_printf("<h2>The server load is currently too high.\n"
+         "Please try again later.</h2>\n"
+         "<p>Current load average: %f<br>\n"
+         "Load average limit: %f<br>\n"
+         "URL: %h%h<br>\n"
+         "Timestamp: %hZ</p>\n",(load_average()),(mxLoad),(g.zBaseURL),(P("PATH_INFO")),(db_text("","SELECT datetime()")));
+  style_finish_page();
+}
+
 /*
 ** Abort the current page request if the load average of the host
 ** computer is too high. Admin and Setup users are exempt from this
@@ -63,14 +83,7 @@ void load_control(void){
     return;
   }
 #endif
-
-  style_set_current_feature("test");
-  style_header("Server Overload");
-  cgi_printf("<h2>The server load is currently too high.\n"
-         "Please try again later.</h2>\n"
-         "<p>Current load average: %f.<br>\n"
-         "Load average limit: %f</p>\n",(load_average()),(mxLoad));
-  style_finish_page();
+  overload_page();
   cgi_set_status(503,"Server Overload");
   cgi_reply();
   exit(0);
