@@ -84,9 +84,8 @@ static void mkdeltaFunc(
   sqlite3_value **argv
 ){
   const char *zFile;
-  Blob x, y;
+  Blob x, y, out;
   int rid;
-  char *aOut;
   int nOut;
   sqlite3_int64 sz;
 
@@ -107,13 +106,8 @@ static void mkdeltaFunc(
     blob_reset(&x);
     return;
   }
-  aOut = sqlite3_malloc64(sz+70);
-  if( aOut==0 ){
-    sqlite3_result_error_nomem(context);
-    blob_reset(&y);
-    blob_reset(&x);
-    return;
-  }
+  blob_init(&out, 0, 0);
+  blob_resize(&out, sz+70);
   if( blob_size(&x)==blob_size(&y)
    && memcmp(blob_buffer(&x), blob_buffer(&y), blob_size(&x))==0
   ){
@@ -123,14 +117,14 @@ static void mkdeltaFunc(
     return;
   }
   nOut = delta_create(blob_buffer(&x),blob_size(&x),
-                      blob_buffer(&y),blob_size(&y), aOut);
+                      blob_buffer(&y),blob_size(&y), blob_buffer(&out));
+  blob_resize(&out, nOut);
   blob_reset(&x);
   blob_reset(&y);
-  blob_init(&x, aOut, nOut);
-  blob_compress(&x, &x);
-  sqlite3_result_blob64(context, blob_buffer(&x), blob_size(&x),
+  blob_compress(&out, &out);
+  sqlite3_result_blob64(context, blob_buffer(&out), blob_size(&out),
                         SQLITE_TRANSIENT);
-  blob_reset(&x);
+  blob_reset(&out);
 }
 
 
@@ -390,7 +384,7 @@ void patch_apply(unsigned mFlags){
       fossil_fatal("Cannot apply patch: there are unsaved changes "
                    "in the current check-out");
     }else{
-      blob_appendf(&cmd, "%$ revert", g.nameOfExe);
+      blob_appendf(&cmd, "%$ revert --noundo", g.nameOfExe);
       if( mFlags & PATCH_DRYRUN ){
         fossil_print("%s\n", blob_str(&cmd));
       }else{
@@ -432,6 +426,7 @@ void patch_apply(unsigned mFlags){
   }
   blob_reset(&cmd);
   if( db_table_exists("patch","patchmerge") ){
+    int nMerge = 0;
     db_prepare(&q,
       "SELECT type, mhash, upper(type) FROM patch.patchmerge"
       " WHERE type IN ('merge','cherrypick','backout','integrate')"
@@ -441,10 +436,13 @@ void patch_apply(unsigned mFlags){
       const char *zType = db_column_text(&q,0);
       blob_append_escaped_arg(&cmd, g.nameOfExe, 1);
       if( strcmp(zType,"merge")==0 ){
-        blob_appendf(&cmd, " merge %s\n", db_column_text(&q,1));
+        blob_appendf(&cmd, " merge --noundo --nosync %s\n",
+                     db_column_text(&q,1));
       }else{
-        blob_appendf(&cmd, " merge --%s %s\n", zType, db_column_text(&q,1));
+        blob_appendf(&cmd, " merge --%s --noundo --nosync %s\n",
+                     zType, db_column_text(&q,1));
       }
+      nMerge++;
       if( mFlags & PATCH_VERBOSE ){
         fossil_print("%-10s %s\n", db_column_text(&q,2),
                     db_column_text(&q,0));
@@ -461,6 +459,41 @@ void patch_apply(unsigned mFlags){
       }
     }
     blob_reset(&cmd);
+
+    /* 2024-12-16 https://fossil-scm.org/home/forumpost/51a37054
+    ** If one or more merge operations occurred in the patch and there are
+    ** files that are marked as "chnged' in the local VFILE but which
+    ** are not mentioned as having been modified in the patch, then
+    ** revert those files.
+    */
+    if( nMerge ){
+      int vid = db_lget_int("checkout", 0);
+      int nRevert = 0;
+      blob_append_escaped_arg(&cmd, g.nameOfExe, 1);
+      blob_appendf(&cmd, " revert --noundo ");
+      db_prepare(&q,
+        "SELECT pathname FROM vfile WHERE vid=%d AND chnged "
+        "EXCEPT SELECT pathname FROM chng",
+        vid
+      );
+      while( db_step(&q)==SQLITE_ROW ){
+        blob_append_escaped_arg(&cmd, db_column_text(&q,0), 1);
+        nRevert++;
+      }
+      db_finalize(&q);
+      if( nRevert ){
+        if( mFlags & PATCH_DRYRUN ){
+          fossil_print("%s", blob_str(&cmd));
+        }else{
+          int rc = fossil_unsafe_system(blob_str(&cmd));
+          if( rc ){
+            fossil_fatal("unable to do reverts:\n%s",
+                         blob_str(&cmd));
+          }
+        }
+      }
+      blob_reset(&cmd);
+    }
   }
 
   /* Deletions */
@@ -669,6 +702,22 @@ static char *patch_find_patch_filename(const char *zCmdName){
   return zPatchFile;
 }
 
+/*
+** Resolves a patch-command remote system name, accounting for patch
+** aliases.
+**
+** If a CONFIG table entry matching name='patch-alias:$zKey' is found,
+** the corresponding value is returned, else a fossil_strdup() of zKey
+** is returned. The caller is responsible for passing the resulting
+** string to fossil_free().
+*/
+static char *patch_resolve_remote(const char *zKey){
+  char *zAlias = db_text(0, "SELECT value FROM config "
+                            "WHERE name = 'patch-alias:%q'",
+                            zKey);
+  return zAlias ? zAlias : fossil_strdup(zKey);
+}
+
 /*
 ** Create a FILE* that will execute the remote side of a push or pull
 ** using ssh (probably) or fossil for local pushes and pulls.  Return
@@ -699,7 +748,7 @@ static FILE *patch_remote_command(
   if( g.argc!=4 ){
     usage(mprintf("%s [USER@]HOST:DIRECTORY", zThisCmd));
   }
-  zRemote = fossil_strdup(g.argv[3]);
+  zRemote = patch_resolve_remote(g.argv[3]);
   zDir = (char*)file_skip_userhost(zRemote);
   if( zDir==0 ){
     if( isRetry ) goto remote_command_error;
@@ -748,7 +797,7 @@ remote_command_error:
 ** by g.argv[3].
 */
 static void patch_toggle_ssh_needs_path(void){
-  char *zRemote = fossil_strdup(g.argv[3]);
+  char *zRemote = patch_resolve_remote(g.argv[3]);
   char *zDir = (char*)file_skip_userhost(zRemote);
   if( zDir ){
     *(char*)(zDir - 1) =  0;
@@ -875,7 +924,6 @@ static void patch_diff(
   if( nErr ) fossil_fatal("abort due to prior errors");
 }
 
-
 /*
 ** COMMAND: patch
 **
@@ -886,6 +934,18 @@ static void patch_diff(
 ** uncommitted changes of a check-out.  Use Fossil binary patches to transfer
 ** proposed or incomplete changes between machines for testing or analysis.
 **
+** > fossil patch alias add|rm|ls|list ?ARGS?
+**
+**       Manage remote-name aliases, which act as short-form
+**       equivalents to REMOTE-CHECKOUT strings. Aliases are local to
+**       a given repository and do not sync. Subcommands:
+**
+**         ... add ALIAS REMOTE-CHECKOUT       Add ALIAS as an alias
+**                                             for REMOTE-CHECKOUT.
+**         ... ls|list                         List all local aliases.
+**         ... rm ALIAS [ALIAS...]             Remove named aliases
+**         ... rm --all                        Remove all aliases
+**
 ** > fossil patch create [DIRECTORY] PATCHFILE
 **
 **       Create a new binary patch in PATCHFILE that captures all uncommitted
@@ -966,10 +1026,72 @@ void patch_cmd(void){
   size_t n;
   if( g.argc<3 ){
     patch_usage:
-    usage("apply|create|diff|gdiff|pull|push|view");
+    usage("alias|apply|create|diff|gdiff|pull|push|view");
   }
   zCmd = g.argv[2];
   n = strlen(zCmd);
+  if( strncmp(zCmd, "alias", n)==0 ){
+    const char * zArg = g.argc>3 ? g.argv[3] : 0;
+    db_must_be_within_tree();
+    if( 0==zArg ){
+      goto usage_patch_alias;
+    }else if( 0==strcmp("ls",zArg) || 0==strcmp("list",zArg) ){
+      /* alias ls|list */
+      Stmt q;
+      int nAlias = 0;
+
+      verify_all_options();
+      db_prepare(&q, "SELECT substr(name,13), value FROM config "
+                 "WHERE name GLOB 'patch-alias:*' ORDER BY name");
+      while( SQLITE_ROW==db_step(&q) ){
+        const char *zName = db_column_text(&q, 0);
+        const char *zVal = db_column_text(&q, 1);
+        ++nAlias;
+        fossil_print("%s = %s\n", zName, zVal);
+      }
+      db_finalize(&q);
+      if( 0==nAlias ){
+        fossil_print("No patch aliases defined\n");
+      }
+    }else if( 0==strcmp("add", zArg) ){
+      /* alias add localName remote */
+      verify_all_options();
+      if( 6!=g.argc ){
+        usage("alias add localName remote");
+      }
+      db_unprotect(PROTECT_CONFIG);
+      db_multi_exec("REPLACE INTO config (name, value, mtime) "
+                    "VALUES ('patch-alias:%q', %Q, unixepoch())",
+                    g.argv[4], g.argv[5]);
+      db_protect_pop();
+    }else if( 0==strcmp("rm", zArg) ){
+      /* alias rm */
+      const int fAll = 0!=find_option("all", 0, 0);
+      if( fAll ? g.argc<4 : g.argc<5 ){
+        usage("alias rm [-all] [aliasGlob [...aliasGlobN]]");
+      }
+      verify_all_options();
+      db_unprotect(PROTECT_CONFIG);
+      if( 0!=fAll ){
+        db_multi_exec("DELETE FROM config WHERE name GLOB 'patch-alias:*'");
+      }else{
+        Stmt q;
+        int i;
+        db_prepare(&q, "DELETE FROM config WHERE name "
+                   "GLOB 'patch-alias:' || :pattern");
+        for(i = 4; i < g.argc; ++i){
+          db_bind_text(&q, ":pattern", g.argv[i]);
+          db_step(&q);
+          db_reset(&q);
+        }
+        db_finalize(&q);
+      }
+      db_protect_pop();
+    }else{
+    usage_patch_alias:
+      usage("alias ls|list|add|rm ...");
+    }
+  }else
   if( strncmp(zCmd, "apply", n)==0 ){
     char *zIn;
     unsigned flags = 0;
@@ -1003,10 +1125,14 @@ void patch_cmd(void){
       return;
     }
     db_find_and_open_repository(0, 0);
+    if( gdiff_using_tk(zCmd[0]=='g') ){
+      diff_tk("patch diff", 3);
+      return;
+    }
     if( find_option("force","f",0) )    flags |= PATCH_FORCE;
     diff_options(&DCfg, zCmd[0]=='g', 0);
     verify_all_options();
-    zIn = patch_find_patch_filename("apply");
+    zIn = patch_find_patch_filename("diff");
     patch_attach(zIn, stdin, 0);
     patch_diff(flags, &DCfg);
     fossil_free(zIn);
