@@ -56,12 +56,15 @@
 ** to p copies of X following by q-p copies of X? and that the size of the
 ** regular expression in the O(N*M) performance bound is computed after
 ** this expansion.
+**
+** To help prevent DoS attacks, the maximum size of the NFA is restricted.
 */
 #include "config.h"
 #include "regexp.h"
 
 /* The end-of-input character */
 #define RE_EOF            0    /* End of input */
+#define RE_START  0xfffffff    /* Start of input - larger than an UTF-8 */
 
 /* The NFA is implemented as sequence of opcodes taken from the following
 ** set.  Each opcode has a single integer argument.
@@ -83,6 +86,7 @@
 #define RE_OP_SPACE      15    /* space:  [ \t\n\r\v\f] */
 #define RE_OP_NOTSPACE   16    /* Not a digit */
 #define RE_OP_BOUNDARY   17    /* Boundary between word and non-word */
+#define RE_OP_ATSTART    18    /* Currently at the start of the string */
 
 /* Each opcode is a "state" in the NFA */
 typedef unsigned short ReStateNumber;
@@ -116,9 +120,10 @@ struct ReCompiled {
   int *aArg;                  /* Arguments to each operator */
   unsigned (*xNextChar)(ReInput*);  /* Next character function */
   unsigned char zInit[12];    /* Initial text to match */
-  int nInit;                  /* Number of characters in zInit */
+  int nInit;                  /* Number of bytes in zInit */
   unsigned nState;            /* Number of entries in aOp[] and aArg[] */
   unsigned nAlloc;            /* Slots allocated for aOp[] and aArg[] */
+  unsigned mxAlloc;           /* Complexity limit */
 };
 #endif
 
@@ -147,7 +152,7 @@ static unsigned re_next_char(ReInput *p){
       c = (c&0x0f)<<12 | ((p->z[p->i]&0x3f)<<6) | (p->z[p->i+1]&0x3f);
       p->i += 2;
       if( c<=0x7ff || (c>=0xd800 && c<=0xdfff) ) c = 0xfffd;
-    }else if( (c&0xf8)==0xf0 && p->i+3<p->mx && (p->z[p->i]&0xc0)==0x80
+    }else if( (c&0xf8)==0xf0 && p->i+2<p->mx && (p->z[p->i]&0xc0)==0x80
            && (p->z[p->i+1]&0xc0)==0x80 && (p->z[p->i+2]&0xc0)==0x80 ){
       c = (c&0x07)<<18 | ((p->z[p->i]&0x3f)<<12) | ((p->z[p->i+1]&0x3f)<<6)
                        | (p->z[p->i+2]&0x3f);
@@ -188,7 +193,7 @@ int re_match(ReCompiled *pRe, const unsigned char *zIn, int nIn){
   ReStateNumber *pToFree;
   unsigned int i = 0;
   unsigned int iSwap = 0;
-  int c = RE_EOF+1;
+  int c = RE_START;
   int cPrev = 0;
   int rc = 0;
   ReInput in;
@@ -207,6 +212,7 @@ int re_match(ReCompiled *pRe, const unsigned char *zIn, int nIn){
       in.i++;
     }
     if( in.i+pRe->nInit>in.mx ) return 0;
+    c = RE_START-1;
   }
 
   if( pRe->nState<=(sizeof(aSpace)/(sizeof(aSpace[0])*2)) ){
@@ -235,6 +241,10 @@ int re_match(ReCompiled *pRe, const unsigned char *zIn, int nIn){
           if( pRe->aArg[x]==c ) re_add_state(pNext, x+1);
           break;
         }
+        case RE_OP_ATSTART: {
+          if( cPrev==RE_START ) re_add_state(pThis, x+1);
+          break;
+        }
         case RE_OP_ANY: {
           if( c!=0 ) re_add_state(pNext, x+1);
           break;
@@ -287,9 +297,9 @@ int re_match(ReCompiled *pRe, const unsigned char *zIn, int nIn){
         }
         case RE_OP_CC_EXC: {
           if( c==0 ) break;
-          /* fall-through */
+          /* fall-through */ goto re_op_cc_inc;
         }
-        case RE_OP_CC_INC: {
+        case RE_OP_CC_INC: re_op_cc_inc: {
           int j = 1;
           int n = pRe->aArg[x];
           int hit = 0;
@@ -316,7 +326,9 @@ int re_match(ReCompiled *pRe, const unsigned char *zIn, int nIn){
     }
   }
   for(i=0; i<pNext->nState; i++){
-    if( pRe->aOp[pNext->aState[i]]==RE_OP_ACCEPT ){ rc = 1; break; }
+    int x = pNext->aState[i];
+    while( pRe->aOp[x]==RE_OP_GOTO ) x += pRe->aArg[x];
+    if( pRe->aOp[x]==RE_OP_ACCEPT ){ rc = 1; break; }
   }
 re_match_end:
   fossil_free(pToFree);
@@ -328,11 +340,12 @@ re_match_end:
 static int re_resize(ReCompiled *p, int N){
   char *aOp;
   int *aArg;
+  if( N>p->mxAlloc ){ p->zErr = "REGEXP pattern too big"; return 1; }
   aOp = fossil_realloc(p->aOp, N*sizeof(p->aOp[0]));
-  if( aOp==0 ) return 1;
+  if( aOp==0 ){ p->zErr = "out of memory"; return 1; }
   p->aOp = aOp;
   aArg = fossil_realloc(p->aArg, N*sizeof(p->aArg[0]));
-  if( aArg==0 ) return 1;
+  if( aArg==0 ){ p->zErr = "out of memory"; return 1; }
   p->aArg = aArg;
   p->nAlloc = N;
   return 0;
@@ -471,7 +484,6 @@ static const char *re_subcompile_string(ReCompiled *p){
     iStart = p->nState;
     switch( c ){
       case '|':
-      case '$':
       case ')': {
         p->sIn.i--;
         return 0;
@@ -508,24 +520,41 @@ static const char *re_subcompile_string(ReCompiled *p){
         re_insert(p, iPrev, RE_OP_FORK, p->nState - iPrev+1);
         break;
       }
+      case '$': {
+        re_append(p, RE_OP_MATCH, RE_EOF);
+        break;
+      }
+      case '^': {
+        re_append(p, RE_OP_ATSTART, 0);
+        break;
+      }
       case '{': {
-        int m = 0, n = 0;
-        int sz, j;
+        unsigned int m = 0, n = 0;
+        unsigned int sz, j;
         if( iPrev<0 ) return "'{m,n}' without operand";
-        while( (c=rePeek(p))>='0' && c<='9' ){ m = m*10 + c - '0'; p->sIn.i++; }
+        while( (c=rePeek(p))>='0' && c<='9' ){
+          m = m*10 + c - '0';
+          if( m*2>p->mxAlloc ) return "REGEXP pattern too big";
+          p->sIn.i++;
+        }
         n = m;
         if( c==',' ){
           p->sIn.i++;
           n = 0;
-          while( (c=rePeek(p))>='0' && c<='9' ){ n = n*10 + c-'0'; p->sIn.i++; }
+          while( (c=rePeek(p))>='0' && c<='9' ){
+            n = n*10 + c-'0';
+            if( n*2>p->mxAlloc ) return "REGEXP pattern too big";
+            p->sIn.i++;
+          }
         }
         if( c!='}' ) return "unmatched '{'";
-        if( n>0 && n<m ) return "n less than m in '{m,n}'";
+        if( n<m ) return "n less than m in '{m,n}'";
         p->sIn.i++;
         sz = p->nState - iPrev;
         if( m==0 ){
           if( n==0 ) return "both m and n are zero in '{m,n}'";
           re_insert(p, iPrev, RE_OP_FORK, sz+1);
+          iPrev++;
           n--;
         }else{
           for(j=1; j<m; j++) re_copy(p, iPrev, sz);
@@ -535,12 +564,12 @@ static const char *re_subcompile_string(ReCompiled *p){
           re_copy(p, iPrev, sz);
         }
         if( n==0 && m>0 ){
-          re_append(p, RE_OP_FORK, -sz);
+          re_append(p, RE_OP_FORK, -(int)sz);
         }
         break;
       }
       case '[': {
-        int iFirst = p->nState;
+        unsigned int iFirst = p->nState;
         if( rePeek(p)=='^' ){
           re_append(p, RE_OP_CC_EXC, 0);
           p->sIn.i++;
@@ -564,7 +593,7 @@ static const char *re_subcompile_string(ReCompiled *p){
           if( rePeek(p)==']' ){ p->sIn.i++; break; }
         }
         if( c==0 ) return "unclosed '['";
-        p->aArg[iFirst] = p->nState - iFirst;
+        if( p->nState>iFirst ) p->aArg[iFirst] = p->nState - iFirst;
         break;
       }
       case '\\': {
@@ -615,7 +644,12 @@ void re_free(ReCompiled *pRe){
 ** compiled regular expression in *ppRe.  Return NULL on success or an
 ** error message if something goes wrong.
 */
-const char *re_compile(ReCompiled **ppRe, const char *zIn, int noCase){
+static const char *re_compile(
+  ReCompiled **ppRe,      /* OUT: write compiled NFA here */
+  const char *zIn,        /* Input regular expression */
+  int mxRe,               /* Complexity limit */
+  int noCase              /* True for caseless comparisons */
+){
   ReCompiled *pRe;
   const char *zErr;
   int i, j;
@@ -627,9 +661,11 @@ const char *re_compile(ReCompiled **ppRe, const char *zIn, int noCase){
   }
   memset(pRe, 0, sizeof(*pRe));
   pRe->xNextChar = noCase ? re_next_char_nocase : re_next_char;
+  pRe->mxAlloc = mxRe;
   if( re_resize(pRe, 30) ){
+    zErr = pRe->zErr;
     re_free(pRe);
-    return "out of memory";
+    return zErr;
   }
   if( zIn[0]=='^' ){
     zIn++;
@@ -644,11 +680,7 @@ const char *re_compile(ReCompiled **ppRe, const char *zIn, int noCase){
     re_free(pRe);
     return zErr;
   }
-  if( rePeek(pRe)=='$' && pRe->sIn.i+1>=pRe->sIn.mx ){
-    re_append(pRe, RE_OP_MATCH, RE_EOF);
-    re_append(pRe, RE_OP_ACCEPT, 0);
-    *ppRe = pRe;
-  }else if( pRe->sIn.i>=pRe->sIn.mx ){
+  if( pRe->sIn.i>=pRe->sIn.mx ){
     re_append(pRe, RE_OP_ACCEPT, 0);
     *ppRe = pRe;
   }else{
@@ -661,19 +693,19 @@ const char *re_compile(ReCompiled **ppRe, const char *zIn, int noCase){
   ** one or more matching characters, enter those matching characters into
   ** zInit[].  The re_match() routine can then search ahead in the input
   ** string looking for the initial match without having to run the whole
-  ** regex engine over the string.  Do not worry able trying to match
+  ** regex engine over the string.  Do not worry about trying to match
   ** unicode characters beyond plane 0 - those are very rare and this is
   ** just an optimization. */
   if( pRe->aOp[0]==RE_OP_ANYSTAR && !noCase ){
     for(j=0, i=1; j<(int)sizeof(pRe->zInit)-2 && pRe->aOp[i]==RE_OP_MATCH; i++){
       unsigned x = pRe->aArg[i];
-      if( x<=127 ){
+      if( x<=0x7f ){
         pRe->zInit[j++] = (unsigned char)x;
-      }else if( x<=0xfff ){
+      }else if( x<=0x7ff ){
         pRe->zInit[j++] = (unsigned char)(0xc0 | (x>>6));
         pRe->zInit[j++] = 0x80 | (x&0x3f);
       }else if( x<=0xffff ){
-        pRe->zInit[j++] = (unsigned char)(0xd0 | (x>>12));
+        pRe->zInit[j++] = (unsigned char)(0xe0 | (x>>12));
         pRe->zInit[j++] = 0x80 | ((x>>6)&0x3f);
         pRe->zInit[j++] = 0x80 | (x&0x3f);
       }else{
@@ -711,11 +743,14 @@ static void re_sql_func(
   if( pRe==0 ){
     zPattern = (const char*)sqlite3_value_text(argv[0]);
     if( zPattern==0 ) return;
-    zErr = re_compile(&pRe, zPattern, sqlite3_user_data(context)!=0);
+    zErr = fossil_re_compile(&pRe, zPattern, sqlite3_user_data(context)!=0);
     if( zErr ){
       re_free(pRe);
+      /* The original SQLite function from which this code was copied raises
+      ** an error if the REGEXP contained a syntax error.  This variant
+      ** silently fails to match, as that works better for Fossil.
+      ** sqlite3_result_error(context, zErr, -1); */
       sqlite3_result_int(context, 0);
-      /* sqlite3_result_error(context, zErr, -1); */
       return;
     }
     if( pRe==0 ){
@@ -739,17 +774,78 @@ static void re_sql_func(
 */
 int re_add_sql_func(sqlite3 *db){
   int rc;
-  rc = sqlite3_create_function(db, "regexp", 2, SQLITE_UTF8|SQLITE_INNOCUOUS,
+  rc = sqlite3_create_function(db, "regexp", 2,
+                           SQLITE_UTF8|SQLITE_INNOCUOUS|SQLITE_DETERMINISTIC,
                                0, re_sql_func, 0, 0);
   if( rc==SQLITE_OK ){
     /* The regexpi(PATTERN,STRING) function is a case-insensitive version
     ** of regexp(PATTERN,STRING). */
-    rc = sqlite3_create_function(db, "regexpi", 2, SQLITE_UTF8|SQLITE_INNOCUOUS,
+    rc = sqlite3_create_function(db, "regexpi", 2,
+                           SQLITE_UTF8|SQLITE_INNOCUOUS|SQLITE_DETERMINISTIC,
                                  (void*)db, re_sql_func, 0, 0);
   }
   return rc;
 }
 
+/*
+** The input zIn is a string that we want to match exactly as part of
+** a regular expression.  Return a new string (in space obtained from
+** fossil_malloc() or the equivalent) that escapes all regexp syntax
+** characters in zIn.
+*/
+char *re_quote(const char *zIn){
+  Blob out;
+  blob_init(&out, 0, 0);
+  while( zIn[0] ){
+    switch( zIn[0] ){
+      case '.':
+      case '?':
+      case '*':
+      case '+':
+      case '\\':
+      case '(':
+      case ')':
+      case '[':
+      case ']':
+      case '|':
+      case '^':
+      case '$':
+      case '{':
+      case '}': {
+        blob_appendf(&out,"\\x%02x", (unsigned char)zIn[0]);
+        break;
+      }
+      default: {
+        blob_append_char(&out, zIn[0]);
+        break;
+      }
+    }
+    zIn++;
+  }
+  blob_materialize(&out);
+  return out.aData;
+}
+
+/*
+** SETTING:  regexp-limit                  width=8 default=1000
+**
+** Limit the size of the bytecode used to implement a regular expression
+** to this many steps.  It is important to limit this to avoid possible
+** DoS attacks.
+*/
+
+/*
+** Compile an RE using re_maxlen().
+*/
+const char *fossil_re_compile(
+  ReCompiled **ppRe,      /* OUT: write compiled NFA here */
+  const char *zIn,        /* Input regular expression */
+  int noCase              /* True for caseless comparisons */
+){
+  int mxLen = g.db ? db_get_int("regexp-limit",1000) : 1000;
+  return re_compile(ppRe, zIn, mxLen, noCase);
+}
+
 /*
 ** Run a "grep" over a single file read from disk.
 */
@@ -814,21 +910,34 @@ static int grep_buffer(
 **
 ** Options:
 **   -i|--ignore-case    Ignore case
+**   --robot-exception   Use the robot-exception setting as the REGEXP
 */
 void re_test_grep(void){
   ReCompiled *pRe;
   const char *zErr;
+  int iFileList = 3;
   int ignoreCase = find_option("ignore-case","i",0)!=0;
-  if( g.argc<3 ){
-    usage("REGEXP [FILE...]");
+  int bRobot = find_option("robot-exception",0,0)!=0;
+  if( bRobot ){
+    const char *zRe;
+    db_find_and_open_repository(0,0);
+    verify_all_options();
+    zRe = db_get("robot-exception","^$");
+    zErr = fossil_re_compile(&pRe, zRe, ignoreCase);
+    iFileList = 2;
+  }else{
+    verify_all_options();
+    if( g.argc<3 ){
+      usage("REGEXP [FILE...]");
+    }
+    zErr = fossil_re_compile(&pRe, g.argv[2], ignoreCase);
   }
-  zErr = re_compile(&pRe, g.argv[2], ignoreCase);
   if( zErr ) fossil_fatal("%s", zErr);
-  if( g.argc==3 ){
+  if( g.argc==iFileList ){
     grep_file(pRe, "-", stdin);
   }else{
     int i;
-    for(i=3; i<g.argc; i++){
+    for(i=iFileList; i<g.argc; i++){
       FILE *in = fossil_fopen(g.argv[i], "rb");
       if( in==0 ){
         fossil_warning("cannot open \"%s\"", g.argv[i]);
@@ -903,7 +1012,7 @@ void re_grep_cmd(void){
   if( g.argc<4 ){
     usage("REGEXP FILENAME ...");
   }
-  zErr = re_compile(&pRe, g.argv[2], ignoreCase);
+  zErr = fossil_re_compile(&pRe, g.argv[2], ignoreCase);
   if( zErr ) fossil_fatal("%s", zErr);
 
   add_content_sql_commands(g.db);
@@ -980,3 +1089,84 @@ void re_grep_cmd(void){
     }
   }
 }
+
+/*
+** WEBPAGE: re_rules
+**
+** Show a summary of the regular expression matching rules for Fossil.
+*/
+void re_rules_page(void){
+  style_set_current_feature("wiki");
+  style_header("Regular Expression Syntax");
+  cgi_printf("<p>Syntax rules for regular expression matching in Fossil:</p>\n"
+         "\n"
+         "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n"
+         "<tr><th>&emsp;&emsp;&emsp;<th>Pattern\n"
+         "    <th>&emsp;&emsp;&emsp;<th align=\"left\">Match\n"
+         "<tr><td><td><i>X</i><b>*</b>\n"
+         "    <td><td>Zero or more occurrences of <i>X</i>\n"
+         "<tr><td><td><i>X</i><b>+</b>\n"
+         "    <td><td>One or more occurrences of <i>X</i>\n"
+         "<tr><td><td><i>X</i><b>?</b>\n"
+         "    <td><td>Zero or one occurrences of <i>X</i>\n"
+         "<tr><td><td><i>X</i><b>{</b><i>P</i><b>,</b><i>Q</i><b>}</b>\n"
+         "    <td><td>Between P and Q occurrences of <i>X</i>\n"
+         "<tr><td><td><b>(</b><i>X</i><b>)</b>\n"
+         "    <td><td><i>X</i>\n"
+         "<tr><td><td><i>X</i><b>|</b><i>Y</i>\n"
+         "    <td><td><i>X</i> or <i>Y</i>\n"
+         "<tr><td><td><b>^</b><i>X</i>\n"
+         "    <td><td><i>X</i> at the beginning of the string\n"
+         "<tr><td><td><i>X</i><b>$</b>\n"
+         "    <td><td><i>X</i> at the end of the string\n"
+         "<tr><td><td><b>.</b>\n"
+         "    <td><td>Any single character\n"
+         "<tr><td><td><b>\\</b><i>C</i>\n"
+         "    <td><td>Character <i>C</i> if <i>C</i> is one of: <b>\\{}()[]|*+?</b>\n"
+         "<tr><td><td><b>\\</b><i>C</i>\n"
+         "    <td><td>C-language escapes if <i>C</i> is one of: <b>afnrtv</b>\n"
+         "<tr><td><td><b>\\u</b><i>HHHH</i>\n"
+         "    <td><td>Unicode character U+HHHH where <i>HHHH</i> is four hex digits\n"
+         "<tr><td><td><b>\\</b><i>HH</i>\n"
+         "    <td><td>Unicode character U+00HH where <i>HH</i> is two hex digits\n"
+         "<tr><td><td><b>[</b><i>abc</i><b>]</b>\n"
+         "    <td><td>Any single character from <i>abc</i>\n"
+         "<tr><td><td><b>[^</b><i>abc</i><b>]</b>\n"
+         "    <td><td>Any single character not in <i>abc</i>\n"
+         "<tr><td><td><b>[</b><i>a-z</i><b>]</b>\n"
+         "    <td><td>Any single character between <i>a</i> and <i>z</i>, inclusive\n"
+         "<tr><td><td><b>[^</b><i>a-z</i><b>]</b>\n"
+         "    <td><td>Any single character not between <i>a</i> and <i>z</i>\n"
+         "<tr><td><td><b>\\b</b>\n"
+         "    <td><td>Word boundary\n"
+         "<tr><td><td><b>\\w</b>\n"
+         "    <td><td>A word character: a-zA-Z0-9 or _\n"
+         "<tr><td><td><b>\\W</b>\n"
+         "    <td><td>A non-word character\n"
+         "<tr><td><td><b>\\d</b>\n"
+         "    <td><td>A digit.  0-9\n"
+         "<tr><td><td><b>\\D</b>\n"
+         "    <td><td>A non-digit character\n"
+         "<tr><td><td><b>\\s</b>\n"
+         "    <td><td>A whitespace character\n"
+         "<tr><td><td><b>\\S</b>\n"
+         "    <td><td>A non-whitespace character\n"
+         "</table>\n"
+         "\n"
+         "<p>In the \"Pattern\" column of the table above:</p>\n"
+         "<ul>\n"
+         "<li> \"<i>X</i>\" and \"<i>Y</i>\" mean any subpattern\n"
+         "<li> \"<i>P</i>\" and \"<i>Q</i>\" mean integers\n"
+         "<li> \"<i>C</i>\" means a single character\n"
+         "<li> \"<i>H</i>\" means a hexadecimal digit\n"
+         "<li> \"<i>abc</i>\" means any sequences of one or more characters\n"
+         "<li> \"<i>a-z</i>\" means any single character, a single \"<b>-</b>\"\n"
+         "     character, and then one additional character.\n"
+         "<li> All other symbols in the patterns are literal text\n"
+         "</ul>\n"
+         "\n"
+         "<p>The \"<i>X</i><b>|</b><i>Y</i>\" pattern has lower precedence\n"
+         "than the others.  Use \"<b>(</b>...<b>)</b>\" for grouping, as\n"
+         "necessary.\n");
+  style_finish_page();
+}
