@@ -586,9 +586,6 @@ void view_edit(void){
   }
   if( zOwner==0 ) zOwner = g.zLogin;
   style_submenu_element("Cancel", "%R/reportlist");
-  if( rn>0 ){
-    style_submenu_element("Delete", "%R/rptedit/%d?del1=1", rn);
-  }
   style_header("%s", rn>0 ? "Edit Report Format":"Create New Report Format");
   if( zErr ){
     cgi_printf("<blockquote class=\"reportError\">%h</blockquote>\n",(zErr));
@@ -900,7 +897,8 @@ static int generate_html(
   ** per database entry.
   */
   if( pState->iNewRow>=0 ){
-    cgi_printf("<tr><td colspan=%d><font size=1>&nbsp;</font></td></tr>\n",(pState->nCol));
+    cgi_printf("<tr><td colspan=\"%d\" style=\"padding:0px\">\n"
+           "<hr style=\"margin:0px\"></td></tr>\n",(pState->nCol));
   }
 
   /* Output the data for this entry from the database
