@@ -103,6 +103,7 @@ void secaudit0_page(void){
   const char *zSelfCap;      /* Capabilities of self-registered users */
   int hasSelfReg = 0;        /* True if able to self-register */
   const char *zPublicUrl;    /* Canonical access URL */
+  const char *zVulnReport;   /* The vuln-report setting */
   Blob cmd;
   char *z;
   int n, i;
@@ -366,6 +367,18 @@ void secaudit0_page(void){
            "of this repository.\n");
   }
 
+  zVulnReport = db_get("vuln-report","log");
+  if( fossil_strcmp(zVulnReport,"block")!=0
+   && fossil_strcmp(zVulnReport,"fatal")!=0
+  ){
+    cgi_printf("<li><p><b>WARNING:</b>\n"
+           "The <a href=\"%R/help?cmd=vuln-report\">vuln-report setting</a>\n"
+           "has a value of \"%h\". This disables defenses against\n"
+           "XSS or SQL-injection vulnerabilities caused by coding errors in\n"
+           "custom TH1 scripts.  For the best security, change\n"
+           "the value of the vuln-report setting to \"block\" or \"fatal\".\n",(zVulnReport));
+  }
+
   /* Obsolete:  */
   if( hasAnyCap(zAnonCap, "d") ||
       hasAnyCap(zDevCap,  "d") ||
@@ -546,7 +559,7 @@ void secaudit0_page(void){
   }
   if( db_get_boolean("http_authentication_ok", 0) ){
     cgi_printf("<li><p><b>Caution:</b>\n"
-           "This repository trusts that the HTTP_AUTHENITICATION environment\n"
+           "This repository trusts that the HTTP_AUTHENTICATION environment\n"
            "variable set up by the webserver contains the name of an\n"
            "authenticated user.\n"
            "Fossil's built-in authentication mechanism is bypassed.\n"
@@ -556,13 +569,13 @@ void secaudit0_page(void){
 
   /* Logging should be turned on
   */
-  if( db_get_boolean("access-log",0)==0 ){
+  if( db_get_boolean("access-log",1)==0 ){
     cgi_printf("<li><p>\n"
            "The <a href=\"access_log\">User Log</a> is disabled.  The user log\n"
            "keeps a record of successful and unsuccessful login attempts and is\n"
            "useful for security monitoring.\n");
   }
-  if( db_get_boolean("admin-log",0)==0 ){
+  if( db_get_boolean("admin-log",1)==0 ){
     cgi_printf("<li><p>\n"
            "The <a href=\"admin_log\">Administrative Log</a> is disabled.\n"
            "The administrative log provides a record of configuration changes\n"
@@ -807,33 +820,63 @@ static void no_error_log_available(void){
   }
 }
 
-/*
-** The maximum number of bytes of the error log to show by default.
-*/
-#define MXSHOWLOG 500000
-
 /*
 ** WEBPAGE: errorlog
 **
 ** Show the content of the error log.  Only the administrator can view
 ** this page.
+**
+**    y=0x001          Show only hack attempts
+**    y=0x002          Show only panics and assertion faults
+**    y=0x004          Show hung backoffice processes
+**    y=0x008          Show POST requests from a different origin
+**    y=0x010          Show SQLITE_AUTH and similar
+**    y=0x020          Show SMTP error reports
+**    y=0x040          Show TH1 vulnerability reports
+**    y=0x080          Show SQL errors
+**    y=0x800          Show other uncategorized messages
+**
+** If y is omitted or is zero, a count of the various message types is
+** shown.
 */
 void errorlog_page(void){
   i64 szFile;
   FILE *in;
   char *zLog;
+  const char *zType = P("y");
+  static const int eAllTypes = 0x8ff;
+  long eType = 0;
+  int bOutput = 0;
+  int prevWasTime = 0;
+  int nHack = 0;
+  int nPanic = 0;
+  int nOther = 0;
+  int nHang = 0;
+  int nXPost = 0;
+  int nAuth = 0;
+  int nSmtp = 0;
+  int nVuln = 0;
+  int nSqlErr = 0;
   char z[10000];
+  char zTime[10000];
+
   login_check_credentials();
   if( !g.perm.Admin ){
     login_needed(0);
     return;
   }
+  if( zType ){
+    eType = strtol(zType,0,0) & eAllTypes;
+  }
   style_header("Server Error Log");
   style_submenu_element("Test", "%R/test-warning");
   style_submenu_element("Refresh", "%R/errorlog");
+  style_submenu_element("Download", "%R/errorlog?download");
+  style_submenu_element("Truncate", "%R/errorlog?truncate");
   style_submenu_element("Log-Menu", "%R/setup-logmenu");
-  style_submenu_element("Panics", "%R/paniclog");
-  style_submenu_element("Non-Hacks", "%R/hacklog?not");
+  if( eType ){
+    style_submenu_element("Summary", "%R/errorlog");
+  }
 
   if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
     no_error_log_available();
@@ -864,79 +907,95 @@ void errorlog_page(void){
   zLog = file_canonical_name_dup(g.zErrlog);
   cgi_printf("<p>The server error log at \"%h\" is %,lld bytes in size.\n",(zLog),(szFile));
   fossil_free(zLog);
-  style_submenu_element("Download", "%R/errorlog?download");
-  style_submenu_element("Truncate", "%R/errorlog?truncate");
   in = fossil_fopen(g.zErrlog, "rb");
   if( in==0 ){
     cgi_printf("<p class='generalError'>Unable to open that file for reading!</p>\n");
     style_finish_page();
     return;
   }
-  if( szFile>MXSHOWLOG && P("all")==0 ){
-    cgi_printf("<form action=\"%R/errorlog\" method=\"POST\">\n"
-           "<p>Only the last %,d bytes are shown.\n"
-           "<input type=\"submit\" name=\"all\" value=\"Show All\">\n"
-           "</form>\n",(MXSHOWLOG));
-    fseek(in, -MXSHOWLOG, SEEK_END);
+  if( eType==0 ){
+    /* will do a summary */
+  }else if( (eType&eAllTypes)!=eAllTypes ){
+    cgi_printf("Only the following types of messages displayed:\n"
+           "<ul>\n");
+    if( eType & 0x01 ){
+      cgi_printf("<li>Hack attempts\n");
+    }
+    if( eType & 0x02 ){
+      cgi_printf("<li>Panics and assertion faults\n");
+    }
+    if( eType & 0x04 ){
+      cgi_printf("<li>Hung backoffice processes\n");
+    }
+    if( eType & 0x08 ){
+      cgi_printf("<li>POST requests from different origin\n");
+    }
+    if( eType & 0x10 ){
+      cgi_printf("<li>SQLITE_AUTH and similar errors\n");
+    }
+    if( eType & 0x20 ){
+      cgi_printf("<li>SMTP malfunctions\n");
+    }
+    if( eType & 0x40 ){
+      cgi_printf("<li>TH1 vulnerabilities\n");
+    }
+    if( eType & 0x80 ){
+      cgi_printf("<li>SQL errors\n");
+    }
+    if( eType & 0x800 ){
+      cgi_printf("<li>Other uncategorized messages\n");
+    }
+    cgi_printf("</ul>\n");
+  }
+  cgi_printf("<hr>\n");
+  if( eType ){
+    cgi_printf("<pre>\n");
   }
-  cgi_printf("<hr>\n"
-         "<pre>\n");
   while( fgets(z, sizeof(z), in) ){
-    cgi_printf("%h",(z));
-  }
-  fclose(in);
-  cgi_printf("</pre>\n");
-  style_finish_page();
-}
-
-/*
-** WEBPAGE: paniclog
-**
-** Scan the error log for panics.  Show all panic messages, ignoring all
-** other error log entries.
-*/
-void paniclog_page(void){
-  i64 szFile;
-  char *zLog;
-  FILE *in;
-  int bOutput = 0;
-  int prevWasTime = 0;
-  char z[10000];
-  char zTime[10000];
-
-  login_check_credentials();
-  if( !g.perm.Admin ){
-    login_needed(0);
-    return;
-  }
-  style_header("Server Panic Log");
-  style_submenu_element("Log-Menu", "%R/setup-logmenu");
-
-  if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
-    no_error_log_available();
-    style_finish_page();
-    return;
-  }
-  in = fossil_fopen(g.zErrlog, "rb");
-  if( in==0 ){
-    cgi_printf("<p class='generalError'>Unable to open that file for reading!</p>\n");
-    style_finish_page();
-    return;
-  }
-  szFile = file_size(g.zErrlog, ExtFILE);
-  zLog = file_canonical_name_dup(g.zErrlog);
-  cgi_printf("Panic messages contained within the %lld-byte \n"
-         "<a href=\"%R/errorlog?all\">error log</a> found at\n"
-         "\"%h\".\n",(szFile),(zLog));
-  fossil_free(zLog);
-  cgi_printf("<hr>\n"
-         "<pre>\n");
-  while( fgets(z, sizeof(z), in) ){
-    if( prevWasTime
-     && (strncmp(z,"panic: ", 7)==0 || strstr(z," assertion fault ")!=0)
-    ){
-      cgi_printf("%h",(zTime));
-      bOutput = 1;
+    if( prevWasTime ){
+      if( strncmp(z,"possible hack attempt - 418 ", 27)==0 ){
+        bOutput = (eType & 0x01)!=0;
+        nHack++;
+      }else
+      if( (strncmp(z,"panic: ", 7)==0 && strncmp(z+7,"Timeout",7)!=0)
+       || strstr(z," assertion fault ")!=0
+      ){
+        bOutput = (eType & 0x02)!=0;
+        nPanic++;
+      }else
+      if( strncmp(z,"SMTP:", 5)==0 ){
+        bOutput = (eType & 0x20)!=0;
+        nSmtp++;
+      }else
+      if( sqlite3_strglob("warning: backoffice process * still *",z)==0 ){
+        bOutput = (eType & 0x04)!=0;
+        nHang++;
+      }else
+      if( sqlite3_strglob("warning: POST from different origin*",z)==0 ){
+        bOutput = (eType & 0x08)!=0;
+        nXPost++;
+      }else
+      if( sqlite3_strglob("SECURITY: authorizer blocks*",z)==0
+       || sqlite3_strglob("warning: SQLITE_AUTH*",z)==0
+      ){
+        bOutput = (eType & 0x10)!=0;
+        nAuth++;
+      }else
+      if( strncmp(z,"possible", 8)==0 && strstr(z,"tainted")!=0 ){
+        bOutput = (eType & 0x40)!=0;
+        nVuln++;
+      }else
+      if( strstr(z,"statement aborts at ") ){
+        bOutput = (eType & 0x80)!=0;
+        nSqlErr++;
+      }else
+      {
+        bOutput = (eType & 0x800)!=0;
+        nOther++;
+      }
+      if( bOutput ){
+        cgi_printf("%h",(zTime));
+      }
     }
     if( strncmp(z, "--------", 8)==0 ){
       size_t n = strlen(z);
@@ -946,79 +1005,61 @@ void paniclog_page(void){
     }else{
       prevWasTime = 0;
     }
-    if( bOutput ){
+    if( bOutput && eType ){
       cgi_printf("%h",(z));
     }
   }
   fclose(in);
-  cgi_printf("</pre>\n");
-  style_finish_page();
-}
-
-/*
-** WEBPAGE: hacklog
-**
-** Scan the error log for "possible hack attempt" entries  Show hack
-** attempt messages only, omitting all others.  Or if the "not" query
-** parameter is present, show only messages that are not hack attempts.
-*/
-void hacklog_page(void){
-  i64 szFile;
-  char *zLog;
-  FILE *in;
-  int bOutput = 0;
-  int prevWasTime = 0;
-  int isNot = P("not")!=0;
-  char z[10000];
-  char zTime[10000];
-
-  login_check_credentials();
-  if( !g.perm.Admin ){
-    login_needed(0);
-    return;
+  if( eType ){
+    cgi_printf("</pre>\n");
   }
-  style_header("Server Hack Log");
-  style_submenu_element("Log-Menu", "%R/setup-logmenu");
-
-  if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
-    no_error_log_available();
-    style_finish_page();
-    return;
-  }
-  in = fossil_fopen(g.zErrlog, "rb");
-  if( in==0 ){
-    cgi_printf("<p class='generalError'>Unable to open that file for reading!</p>\n");
-    style_finish_page();
-    return;
-  }
-  szFile = file_size(g.zErrlog, ExtFILE);
-  zLog = file_canonical_name_dup(g.zErrlog);
-  cgi_printf("%s messages contained within the %lld-byte \n"
-         "<a href=\"%R/errorlog?all\">error log</a> found at\n"
-         "\"%h\".\n",(isNot?"Non-hack":"Hack"),(szFile),(zLog));
-  fossil_free(zLog);
-  cgi_printf("<hr>\n"
-         "<pre>\n");
-  while( fgets(z, sizeof(z), in) ){
-    if( prevWasTime 
-     && ((strncmp(z,"possible hack attempt - 418 ", 27)==0) ^ isNot)
-    ){
-      cgi_printf("%h",(zTime));
-      bOutput = 1;
+  if( eType==0 ){
+    int nNonHack = nPanic + nHang + nAuth + nSmtp + nVuln + nOther + nSqlErr;
+    int nTotal = nNonHack + nHack + nXPost;
+    cgi_printf("<p><table border=\"a\" cellspacing=\"0\" cellpadding=\"5\">\n");
+    if( nPanic>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=2\">Panics</a></td>\n",(nPanic));
     }
-    if( strncmp(z, "--------", 8)==0 ){
-      size_t n = strlen(z);
-      memcpy(zTime, z, n+1);
-      prevWasTime = 1;
-      bOutput = 0;
+    if( nVuln>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=64\">TH1 Vulnerabilities</a></td>\n",(nVuln));
+    }
+    if( nHack>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=1\">Hack Attempts</a></td>\n",(nHack));
+    }
+    if( nSqlErr>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=128\">SQL Errors</a></td>\n",(nSqlErr));
+    }
+    if( nHang>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=4\">Hung Backoffice</a></td>\n",(nHang));
+    }
+    if( nXPost>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=8\">POSTs from different origin</a></td>\n",(nXPost));
+    }
+    if( nAuth>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=16\">SQLITE_AUTH and similar</a></td>\n",(nAuth));
+    }
+    if( nSmtp>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=32\">SMTP faults</a></td>\n",(nSmtp));
+    }
+    if( nOther>0 ){
+      cgi_printf("<tr><td align=\"right\">%d</td>\n"
+             "    <td><a href=\"./errorlog?y=2048\">Other</a></td>\n",(nOther));
+    }
+    cgi_printf("<tr><td align=\"right\">%d</td>\n",(nTotal));
+    if( nTotal>0 ){
+      cgi_printf("    <td><a href=\"./errorlog?y=4095\">All Messages</a></td>\n");
     }else{
-      prevWasTime = 0;
-    }
-    if( bOutput ){
-      cgi_printf("%h",(z));
+      cgi_printf("    <td>All Messages</td>\n");
     }
+    cgi_printf("</table>\n");
   }
-  fclose(in);
-  cgi_printf("</pre>\n");
   style_finish_page();
 }
