@@ -376,7 +376,7 @@ void rcvfromlist_page(void){
     login_needed(0);
     return;
   }
-  style_header("Artifact Receipts");
+  style_header("Xfer Log");
   style_submenu_element("Log-Menu", "setup-logmenu");
   if( showAll ){
     ofst = 0;
@@ -418,8 +418,9 @@ void rcvfromlist_page(void){
     showAll ? -1 : perScreen+1, ofst
   );
   cgi_printf("<p>Whenever new artifacts are added to the repository, either by\n"
-         "push or using the web interface, an entry is made in the RCVFROM table\n"
-         "to record the source of that artifact.  This log facilitates\n"
+         "push or using the web interface or by \"fossil commit\" or similar,\n"
+         "an entry is made in the RCVFROM table\n"
+         "to record the source of those artifacts.  This log facilitates\n"
          "finding and fixing attempts to inject illicit content into the\n"
          "repository.</p>\n"
          "\n"
