@@ -262,6 +262,7 @@ static int stash_create(void){
   int vid;                           /* Current check-out */
 
   zComment = find_option("comment", "m", 1);
+  (void)fossil_text_editor();
   verify_all_options();
   if( zComment==0 ){
     Blob prompt;                       /* Prompt for stash comment */
@@ -511,8 +512,8 @@ static int stash_get_id(const char *zStashId){
 ** Usage: %fossil stash SUBCOMMAND ARGS...
 **
 ** > fossil stash
-** > fossil stash save ?-m|--comment COMMENT? ?FILES...?
-** > fossil stash snapshot ?-m|--comment COMMENT? ?FILES...?
+** > fossil stash save ?FILES...?
+** > fossil stash snapshot ?FILES...?
 **
 **      Save the current changes in the working tree as a new stash.
 **      Then revert the changes back to the last check-in.  If FILES
@@ -521,6 +522,11 @@ static int stash_get_id(const char *zStashId){
 **      arguments.  The "snapshot" verb works the same as "save" but
 **      omits the revert, keeping the check-out unchanged.
 **
+**      Options:
+**         --editor NAME                  Use the NAME editor to enter comment
+**         -m|--comment COMMENT           Comment text for the new stash
+**
+**
 ** > fossil stash list|ls ?-v|--verbose? ?-W|--width NUM?
 **
 **      List all changes sets currently stashed.  Show information about
@@ -559,6 +565,10 @@ static int stash_get_id(const char *zStashId){
 **      Show diffs of the current working directory and what that
 **      directory would be if STASHID were applied. With gdiff,
 **      gdiff-command is used instead of internal diff logic.
+**
+** > fossil stash rename STASHID NEW-NAME
+**
+**      Change the description of the given STASHID entry to NEW-NAME.
 */
 void stash_cmd(void){
   const char *zCmd;
@@ -759,7 +769,7 @@ void stash_cmd(void){
     if( strstr(zCmd,"show")!=0 || strstr(zCmd,"cat")!=0 ){
       fBaseline = 1;
     }
-    if( find_option("tk",0,0)!=0 ){
+    if( find_option("tk",0,0)!=0 || gdiff_using_tk(zCmd[0]=='g') ){
       db_close(0);
       diff_tk(fBaseline ? "stash show" : "stash diff", 3);
       return;
@@ -768,7 +778,13 @@ void stash_cmd(void){
     stashid = stash_get_id(g.argc==4 ? g.argv[3] : 0);
     stash_diff(stashid, fBaseline, &DCfg);
   }else
-  if( strncmp(zCmd, "help", nCmd)==0 ){
+  if( strncmp(zCmd, "rename", nCmd)==0 ){
+    if( g.argc!=5 ) usage("rename STASHID NAME");
+    stashid = stash_get_id(g.argv[3]);
+    db_multi_exec("UPDATE STASH SET COMMENT=%Q WHERE stashid=%d",
+                  g.argv[4], stashid);
+  }
+  else if( strncmp(zCmd, "help", nCmd)==0 ){
     g.argv[1] = "help";
     g.argv[2] = "stash";
     g.argc = 3;
