@@ -169,7 +169,7 @@ void stat_page(void){
     style_submenu_element("Table Sizes", "repo-tabsize");
   }
   if( g.perm.Admin || g.perm.Setup || db_get_boolean("test_env_enable",0) ){
-    style_submenu_element("Environment", "test_env");
+    style_submenu_element("Environment", "test-env");
   }
   cgi_printf("<table class=\"label-value\">\n");
   fsize = file_size(g.zRepositoryName, ExtFILE);
@@ -271,13 +271,15 @@ void stat_page(void){
   z = db_text(0, "SELECT timediff('now',(SELECT min(mtime) FROM event));");
   sscanf(z, "+%d-%d-%d", &Y, &M, &D);
   if( Y>0 ){
-    cgi_printf("%d years, ",(Y));
+    cgi_printf("%d year%s ",(Y),(Y==1?"":"s"));
   }
   if( M>0 ){
-    cgi_printf("%d months, ",(M));
+    cgi_printf("%d month%s ",(M),(M==1?"":"s"));
   }
-  cgi_printf("%d days\n"
-         "</td></tr>\n",(D));
+  if( D>0 || (Y==0 && M==0) ){
+    cgi_printf("%d day%s\n",(D),(D==1?"":"s"));
+  }
+  cgi_printf("</td></tr>\n");
   p = db_get("project-code", 0);
   if( p ){
     cgi_printf("<tr><th>Project&nbsp;ID:</th>\n"
