@@ -481,7 +481,7 @@ static void image_url_var(const char *zImageName){
 ** Javascript module, and generates HTML elements with the following IDs:
 **
 **    TARGETID:       The <span> wrapper around TEXT.
-**    copy-TARGETID:  The <span> for the copy button.
+**    copy-TARGETID:  The <button> for the copy button.
 **
 ** If the FLIPPED argument is non-zero, the copy button is displayed after TEXT.
 **
@@ -513,14 +513,16 @@ char *style_copy_button(
   if( !bFlipped ){
     const char *zBtnFmt =
       "<span class=\"nobr\">"
-      "<span "
-      "class=\"copy-button\" "
-      "id=\"copy-%h\" "
-      "data-copytarget=\"%h\" "
-      "data-copylength=\"%d\">"
-      "</span>"
+      "<button "
+          "class=\"copy-button\" "
+          "id=\"copy-%h\" "
+          "data-copytarget=\"%h\" "
+          "data-copylength=\"%d\">"
+        "<span>"
+        "</span>"
+      "</button>"
       "<span id=\"%h\">"
-      "%s"
+        "%s"
       "</span>"
       "</span>";
     if( bOutputCGI ){
@@ -536,14 +538,16 @@ char *style_copy_button(
     const char *zBtnFmt =
       "<span class=\"nobr\">"
       "<span id=\"%h\">"
-      "%s"
-      "</span>"
-      "<span "
-      "class=\"copy-button copy-button-flipped\" "
-      "id=\"copy-%h\" "
-      "data-copytarget=\"%h\" "
-      "data-copylength=\"%d\">"
+        "%s"
       "</span>"
+      "<button "
+          "class=\"copy-button copy-button-flipped\" "
+          "id=\"copy-%h\" "
+          "data-copytarget=\"%h\" "
+          "data-copylength=\"%d\">"
+        "<span>"
+        "</span>"
+      "</button>"
       "</span>";
     if( bOutputCGI ){
       cgi_printf(
@@ -747,9 +751,10 @@ static void style_init_th1_vars(const char *zTitle){
   Th_MaybeStore("default_csp", zDfltCsp);
   fossil_free(zDfltCsp);
   Th_Store("nonce", zNonce);
-  Th_Store("project_name", db_get("project-name","Unnamed Fossil Project"));
-  Th_Store("project_description", db_get("project-description",""));
-  if( zTitle ) Th_Store("title", zTitle);
+  Th_StoreUnsafe("project_name",
+                 db_get("project-name","Unnamed Fossil Project"));
+  Th_StoreUnsafe("project_description", db_get("project-description",""));
+  if( zTitle ) Th_Store("title", html_lookalike(zTitle,-1));
   Th_Store("baseurl", g.zBaseURL);
   Th_Store("secureurl", fossil_wants_https(1)? g.zHttpsURL: g.zBaseURL);
   Th_Store("home", g.zTop);
@@ -775,7 +780,7 @@ static void style_init_th1_vars(const char *zTitle){
   image_url_var("logo");
   image_url_var("background");
   if( !login_is_nobody() ){
-    Th_Store("login", g.zLogin);
+    Th_Store("login", html_lookalike(g.zLogin,-1));
   }
   Th_MaybeStore("current_feature", feature_from_page_path(local_zCurrentPage) );
   if( g.ftntsIssues[0] || g.ftntsIssues[1] ||
@@ -824,6 +829,7 @@ void style_header(const char *zTitleFormat, ...){
     cgi_print_all(0, 0, 0);
     cgi_printf("</div>\n");
   }
+  fossil_free(zTitle);
 }
 
 #if INTERFACE
@@ -1303,6 +1309,7 @@ void page_style_css(void){
   image_url_var("logo");
   image_url_var("background");
   Th_Render(blob_str(&css));
+  blob_reset(&css);
 
   /* Tell CGI that the content returned by this page is considered cacheable */
   g.isConst = 1;
@@ -1343,7 +1350,41 @@ static char *find_anon_capabilities(char *zCap){
 }
 
 /*
-** WEBPAGE: test_env
+** WEBPAGE: test-title
+**
+** Render a test page in which the page title is set by the "title"
+** query parameter.  This can be used to show that HTML or Javascript
+** content in the title does not leak through into generated page, resulting
+** in an XSS issue.
+**
+** Due to the potential for abuse, this webpage is only available to
+** administrators.
+*/
+void page_test_title(void){
+  const char *zTitle;
+  login_check_credentials();
+  if( !g.perm.Admin ){
+    login_needed(0);
+  }
+  zTitle = P("title");
+  if( zTitle==0 ){
+    zTitle = "(No Title)";
+  }
+  style_header("%s", zTitle);
+  cgi_printf("<p>\n"
+         "This page sets its title to the value of the \"title\" query parameter.\n"
+         "The form below is a convenient way to set the title query parameter:\n"
+         "\n"
+         "<form method=\"GET\">\n"
+         "Title: <input type=\"text\" size=\"50\" name=\"title\" value=\"%h\">\n"
+         "<input type=\"submit\" value=\"Submit\">\n"
+         "</form>\n",(zTitle));
+  style_finish_page();
+}
+
+/*
+** WEBPAGE: test-env
+** WEBPAGE: test_env  alias
 **
 ** Display CGI-variables and other aspects of the run-time
 ** environment, for debugging and trouble-shooting purposes.
@@ -1352,51 +1393,6 @@ void page_test_env(void){
   webpage_error("");
 }
 
-/*
-** WEBPAGE: honeypot
-** This page is a honeypot for spiders and bots.
-*/
-void honeypot_page(void){
-  unsigned int uSeed = captcha_seed();
-  const char *zDecoded = captcha_decode(uSeed, 0);
-  int bAutoCaptcha = db_get_boolean("auto-captcha", 0);
-  char *zCaptcha = captcha_render(zDecoded);
-  style_header("I think you are a robot");
-  cgi_printf("<p>You seem like a robot.</p>\n"
-         "\n"
-         "<p>Is that incorrect?  Are you really human?\n"
-         "If so, please prove it by transcribing the captcha text\n"
-         "into the entry box below and pressing \"Submit\".\n"
-         "<form action=\"%R/login\" method=\"post\">\n"
-         "<input type=\"hidden\" id=\"u\" name=\"u\" value=\"anonymous\">\n"
-         "<p>\n"
-         "Captcha: <input type=\"text\" id=\"p\" name=\"p\" value=\"\">\n"
-         "<input type=\"submit\" name=\"in\" value=\"Submit\">\n"
-         "\n"
-         "<p>Alternatively, you can <a href=\"%R/login\">log in</a> using an\n"
-         "existing userid.\n"
-         "\n"
-         "<p><input type=\"hidden\" name=\"cs\" value=\"%u\">\n"
-         "<div class=\"captcha\"><table class=\"captcha\"><tr><td>"
-         "<pre class=\"captcha\">\n"
-         "%h\n"
-         "</pre></td></tr></table>\n",(uSeed),(zCaptcha));
-  if( bAutoCaptcha ) {
-     cgi_printf("<input type=\"button\" value=\"Fill out captcha\" id='autofillButton' "
-            "data-af='%s'>\n",(zDecoded));
-     builtin_request_js("login.js");
-  }
-  cgi_printf("</div>\n");
-  free(zCaptcha);
-  cgi_printf("\n"
-         "<p>We regret this inconvenience. However, robots have become so\n"
-         "prolific and so aggressive that they will soak up too much CPU time\n"
-         "and network bandwidth on our servers if allowed to run unchecked.\n"
-         "Your cooperation in demonstrating that you are human is\n"
-         "appreciated.\n");
-  style_finish_page();
-}
-
 /*
 ** Webpages that encounter an error due to missing or incorrect
 ** query parameters can jump to this routine to render an error
@@ -1406,7 +1402,7 @@ void honeypot_page(void){
 ** details of the request environment are displayed.  Otherwise, just
 ** the error message is shown.
 **
-** If zFormat is an empty string, then this is the /test_env page.
+** If zFormat is an empty string, then this is the /test-env page.
 */
 void webpage_error(const char *zFormat, ...){
   int showAll = 0;
@@ -1449,8 +1445,8 @@ void webpage_error(const char *zFormat, ...){
            "g.zPath = %h<br>\n"
            "g.userUid = %d<br>\n"
            "g.zLogin = %h<br>\n"
-           "g.isHuman = %d<br>\n"
-           "g.jsHref = %d<br>\n",(g.zBaseURL),(g.zHttpsURL),(g.zTop),(g.zPath),(g.userUid),(g.zLogin),(g.isHuman),(g.jsHref));
+           "g.isRobot = %d<br>\n"
+           "g.jsHref = %d<br>\n",(g.zBaseURL),(g.zHttpsURL),(g.zTop),(g.zPath),(g.userUid),(g.zLogin),(g.isRobot),(g.jsHref));
     if( g.zLocalRoot ){
       cgi_printf("g.zLocalRoot = %h<br>\n",(g.zLocalRoot));
     }else{
@@ -1506,7 +1502,7 @@ void webpage_error(const char *zFormat, ...){
     P("HTTP_USER_AGENT");
     P("SERVER_SOFTWARE");
     cgi_print_all(showAll, 0, 0);
-    cgi_printf("<p><form method=\"POST\" action=\"%R/test_env\">\n"
+    cgi_printf("<p><form method=\"POST\" action=\"%R/test-env\">\n"
            "<input type=\"hidden\" name=\"showall\" value=\"%d\">\n"
            "<input type=\"submit\" name=\"post-test-button\" value=\"POST Test\">\n"
            "</form>\n",(showAll));
