@@ -128,7 +128,7 @@ static void tktsetup_generic(
     return;
   }
   style_set_current_feature("tktsetup");
-  if( PB("setup") ){
+  if( P("setup") ){
     cgi_redirect("tktsetup");
   }
   isSubmit = P("submit")!=0;
@@ -167,6 +167,7 @@ static void tktsetup_generic(
          "<blockquote><pre>\n"
          "%h\n"
          "</pre></blockquote>\n",(zDesc),(height),(z),(zTitle),(zDfltValue));
+  style_submenu_element("Back", "%R/tktsetup");
   style_finish_page();
 }
 
@@ -304,7 +305,7 @@ void tktsetup_change_page(void){
 static const char zDefaultNew[] =
 "<th1>\n"
 "  if {![info exists mutype]} {set mutype Markdown}\n"
-"  if {[info exists submit]} {\n"
+"  if {[info exists submit] || [info exists submitandnew]} {\n"
 "     set status Open\n"
 "     if {$mutype eq \"HTML\"} {\n"
 "       set mimetype \"text/html\"\n"
@@ -352,6 +353,24 @@ static const char zDefaultNew[] =
 "affect the operation of the product?</td>\n"
 "</tr>\n"
 "\n"
+"<th1>\n"
+"  if {[capexpr {w}]} {\n"
+"     html {<tr><td class=\"tktDspLabel\">Priority:</td><td>}\n"
+"     combobox priority $priority_choices 1\n"
+"     html {\n"
+"       <td align=\"left\">How important is the affected functionality?</td>\n"
+"       </td></tr>\n"
+"     }\n"
+"\n"
+"     html {<tr><td class=\"tktDspLabel\">Subsystem:</td><td>}\n"
+"     combobox subsystem $subsystem_choices 1\n"
+"     html {\n"
+"       <td align=\"left\">Which subsystem is affected?</td>\n"
+"       </td></tr>\n"
+"     }\n"
+"  }\n"
+"</th1>\n"
+"\n"
 "<tr>\n"
 "<td align=\"right\">EMail:</td>\n"
 "<td align=\"left\">\n"
@@ -408,7 +427,15 @@ static const char zDefaultNew[] =
 "<input type=\"submit\" name=\"submit\" value=\"Submit\">\n"
 "</td>\n"
 "<td align=\"left\">After filling in the information above, press this\n"
-"button to create the new ticket</td>\n"
+"button to create the new ticket.</td>\n"
+"</tr>\n"
+"\n"
+"<tr>\n"
+"<td><td align=\"left\">\n"
+"<input type=\"submit\" name=\"submitandnew\" value=\"Submit and New\">\n"
+"</td>\n"
+"<td align=\"left\">Create the new ticket and start another\n"
+"ticket form with the inputs.</td>\n"
 "</tr>\n"
 "<th1>enable_output 1</th1>\n"
 "\n"
@@ -416,7 +443,7 @@ static const char zDefaultNew[] =
 "<td><td align=\"left\">\n"
 "<input type=\"submit\" name=\"cancel\" value=\"Cancel\">\n"
 "</td>\n"
-"<td>Abandon and forget this ticket</td>\n"
+"<td>Abandon and forget this ticket.</td>\n"
 "</tr>\n"
 "</table>\n"
 ;
@@ -457,7 +484,7 @@ static const char zDefaultView[] =
 "  html \"<td class='tktDspValue' colspan='3'>\"\n"
 "  copybtn hash-tk 0 $tkt_uuid 2\n"
 "  if {[hascap s]} {\n"
-"    html \" ($tkt_id)\"\n"
+"    puts \" ($tkt_id)\"\n"
 "  }\n"
 "  html \"</td></tr>\\n\"\n"
 "} else {\n"
@@ -468,6 +495,13 @@ static const char zDefaultView[] =
 "    html \"<td class='tktDspValue' colspan='3'>Deleted</td></tr>\\n\"\n"
 "  }\n"
 "}\n"
+"\n"
+"if {[capexpr {n}]} {\n"
+"  submenu link \"Copy Ticket\" /tktnew/$tkt_uuid\n"
+"}\n"
+"if {[capexpr {nk}]} {\n"
+"  submenu link \"Edit Wiki\" /wikiedit?name=ticket/$tkt_uuid\n"
+"}\n"
 "</th1>\n"
 "<tr><td class=\"tktDspLabel\">Title:</td>\n"
 "<td class=\"tktDspValue\" colspan=\"3\">\n"
@@ -494,22 +528,68 @@ static const char zDefaultView[] =
 "<tr><td class=\"tktDspLabel\">Last&nbsp;Modified:</td><td class=\"tktDspValue\">\n"
 "<th1>\n"
 "if {[info exists tkt_datetime]} {\n"
-"  html $tkt_datetime\n"
+"  puts $tkt_datetime\n"
+"}\n"
+"if {[info exists tkt_mage]} {\n"
+"  html \"<br>[htmlize $tkt_mage] ago\"\n"
 "}\n"
 "</th1>\n"
 "</td>\n"
+"<td class=\"tktDspLabel\">Created:</td><td class=\"tktDspValue\">\n"
+"<th1>\n"
+"if {[info exists tkt_datetime_creation]} {\n"
+"  puts $tkt_datetime_creation\n"
+"}\n"
+"if {[info exists tkt_cage]} {\n"
+"  html \"<br>[htmlize $tkt_cage] ago\"\n"
+"}\n"
+"</th1>\n"
+"</td></tr>\n"
 "<th1>enable_output [hascap e]</th1>\n"
-"  <td class=\"tktDspLabel\">Contact:</td><td class=\"tktDspValue\">\n"
+"  <tr>\n"
+"  <td class=\"tktDspLabel\">Contact:</td><td class=\"tktDspValue\" colspan=\"3\">\n"
 "  $<private_contact>\n"
 "  </td>\n"
+"  </tr>\n"
 "<th1>enable_output 1</th1>\n"
-"</tr>\n"
 "<tr><td class=\"tktDspLabel\">Version&nbsp;Found&nbsp;In:</td>\n"
 "<td colspan=\"3\" valign=\"top\" class=\"tktDspValue\">\n"
-"$<foundin>\n"
+"<th1>\n"
+"set versionlink \"\"\n"
+"set urlfoundin [httpize $foundin]\n"
+"set tagpattern {^[-0-9A-Za-z_\\\\.]+$}\n"
+"if [regexp $tagpattern $foundin] {\n"
+"  query {SELECT count(*) AS match FROM tag\n"
+"         WHERE tagname=concat('sym-',$foundin)} {\n"
+"    if {$match} {set versionlink \"timeline?t=$urlfoundin\"}\n"
+"  }\n"
+"}\n"
+"set hashpattern {^[0-9a-f]+$}\n"
+"if [regexp $hashpattern $foundin] {\n"
+"  set pattern $foundin*\n"
+"  query {SELECT count(*) AS match FROM blob WHERE uuid GLOB $pattern} {\n"
+"    if {$match} {set versionlink \"info/$urlfoundin\"}\n"
+"  }\n"
+"}\n"
+"if {$versionlink eq \"\"} {\n"
+"  puts $foundin\n"
+"} else {\n"
+"  html \"<a href=\\\"\"\n"
+"  puts $versionlink\n"
+"  html \"\\\">\"\n"
+"  puts $foundin\n"
+"  html \"</a>\"\n"
+"}\n"
+"</th1>\n"
 "</td></tr>\n"
+"</table>\n"
 "\n"
 "<th1>\n"
+"wiki_assoc \"ticket\" $tkt_uuid\n"
+"</th1>\n"
+"\n"
+"<table cellpadding=\"5\" style=\"min-width:100%\">\n"
+"<th1>\n"
 "if {[info exists comment]} {\n"
 "  if {[string length $comment]>10} {\n"
 "    html {\n"
@@ -526,7 +606,7 @@ static const char zDefaultView[] =
 "}\n"
 "set seenRow 0\n"
 "set alwaysPlaintext [info exists plaintext]\n"
-"query {SELECT datetime(tkt_mtime) AS xdate, login AS xlogin,\n"
+"query {SELECT datetime(tkt_mtime,toLocal()) AS xdate, login AS xlogin,\n"
 "              mimetype as xmimetype, icomment AS xcomment,\n"
 "              username AS xusername\n"
 "         FROM ticketchng\n"
@@ -534,20 +614,21 @@ static const char zDefaultView[] =
 "  if {$seenRow} {\n"
 "    html \"<hr>\\n\"\n"
 "  } else {\n"
-"    html \"<tr><td class='tktDspLabel'>User Comments:</td></tr>\\n\"\n"
+"    html \"<tr><td class='tktDspLabel' style='text-align:left'>\\n\"\n"
+"    html \"User Comments:</td></tr>\\n\"\n"
 "    html \"<tr><td colspan='5' class='tktDspValue'>\\n\"\n"
 "    set seenRow 1\n"
 "  }\n"
 "  html \"<span class='tktDspCommenter'>\"\n"
-"  html \"[htmlize $xlogin]\"\n"
+"  puts $xlogin\n"
 "  if {$xlogin ne $xusername && [string length $xusername]>0} {\n"
-"    html \" (claiming to be [htmlize $xusername])\"\n"
+"    puts \" (claiming to be $xusername)\"\n"
 "  }\n"
-"  html \" added on $xdate:\"\n"
+"  puts \" added on $xdate:\"\n"
 "  html \"</span>\\n\"\n"
 "  if {$alwaysPlaintext || $xmimetype eq \"text/plain\"} {\n"
 "    set r [randhex]\n"
-"    if {$xmimetype ne \"text/plain\"} {html \"([htmlize $xmimetype])\\n\"}\n"
+"    if {$xmimetype ne \"text/plain\"} {puts \"($xmimetype)\\n\"}\n"
 "    wiki \"<verbatim-$r>[string trimright $xcomment]</verbatim-$r>\\n\"\n"
 "  } elseif {$xmimetype eq \"text/x-fossil-wiki\"} {\n"
 "    wiki \"<p>\\n[string trimright $xcomment]\\n</p>\\n\"\n"
@@ -708,6 +789,48 @@ static const char zDefaultEdit[] =
 "<td>Abandon this edit</td>\n"
 "</tr>\n"
 "\n"
+"<th1>\n"
+"set seenRow 0\n"
+"set alwaysPlaintext [info exists plaintext]\n"
+"query {SELECT datetime(tkt_mtime,toLocal()) AS xdate, login AS xlogin,\n"
+"              mimetype as xmimetype, icomment AS xcomment,\n"
+"              username AS xusername\n"
+"         FROM ticketchng\n"
+"        WHERE tkt_id=$tkt_id AND length(icomment)>0} {\n"
+"  if {$seenRow} {\n"
+"    html \"<hr>\\n\"\n"
+"  } else {\n"
+"    html \"<tr><td colspan='2'><hr></td></tr>\\n\"\n"
+"    html \"<tr><td colspan='2' class='tktDspLabel' style='text-align:left'>\\n\"\n"
+"    html \"Previous User Comments:</td></tr>\\n\"\n"
+"    html \"<tr><td colspan='2' class='tktDspValue'>\\n\"\n"
+"    set seenRow 1\n"
+"  }\n"
+"  html \"<span class='tktDspCommenter'>\"\n"
+"  puts $xlogin\n"
+"  if {$xlogin ne $xusername && [string length $xusername]>0} {\n"
+"    puts \" (claiming to be $xusername)\"\n"
+"  }\n"
+"  puts \" added on $xdate:\"\n"
+"  html \"</span>\\n\"\n"
+"  if {$alwaysPlaintext || $xmimetype eq \"text/plain\"} {\n"
+"    set r [randhex]\n"
+"    if {$xmimetype ne \"text/plain\"} {puts \"($xmimetype)\\n\"}\n"
+"    wiki \"<verbatim-$r>[string trimright $xcomment]</verbatim-$r>\\n\"\n"
+"  } elseif {$xmimetype eq \"text/x-fossil-wiki\"} {\n"
+"    wiki \"<p>\\n[string trimright $xcomment]\\n</p>\\n\"\n"
+"  } elseif {$xmimetype eq \"text/x-markdown\"} {\n"
+"    html [lindex [markdown $xcomment] 1]\n"
+"  } elseif {$xmimetype eq \"text/html\"} {\n"
+"    wiki \"<p><nowiki>\\n[string trimright $xcomment]\\n</nowiki>\\n\"\n"
+"  } else {\n"
+"    set r [randhex]\n"
+"    wiki \"<verbatim-$r links>[string trimright $xcomment]</verbatim-$r>\\n\"\n"
+"  }\n"
+"}\n"
+"if {$seenRow} {html \"</td></tr>\\n\"}\n"
+"</th1>\n"
+"\n"
 "</table>\n"
 ;
 
@@ -806,7 +929,8 @@ static char zDefaultReport[] =
 "       WHEN status='Deferred' THEN '#cacae5'\n"
 "       ELSE '#c8c8c8' END AS 'bgcolor',\n"
 "  substr(tkt_uuid,1,10) AS '#',\n"
-"  datetime(tkt_mtime) AS 'mtime',\n"
+"  datetime(tkt_ctime,toLocal()) AS 'created',\n"
+"  datetime(tkt_mtime,toLocal()) AS 'modified',\n"
 "  type,\n"
 "  status,\n"
 "  subsystem,\n"
@@ -940,6 +1064,7 @@ void tktsetup_timeline_page(void){
          "</p>\n"
          "</div></form>\n");
   db_end_transaction(0);
+  style_submenu_element("Back", "%R/tktsetup");
   style_finish_page();
 
 }
