@@ -669,28 +669,43 @@ int fossil_all_whitespace(const char *z){
 ** not found.
 **
 ** Search algorithm:
-** (1) The local "editor" setting
-** (2) The global "editor" setting
-** (3) The VISUAL environment variable
-** (4) The EDITOR environment variable
-** (5) (Windows only:) "notepad.exe"
+** (1) The value of the --editor command-line argument
+** (2) The local "editor" setting
+** (3) The global "editor" setting
+** (4) The VISUAL environment variable
+** (5) The EDITOR environment variable
+** (6) Any of the following programs that are available:
+**        notepad, nano, pico, jove, edit, vi, vim, ed,
+**
+** The search only occurs once, the first time this routine is called.
+** Second and subsequent invocations always return the same value.
 */
 const char *fossil_text_editor(void){
-  const char *zEditor = db_get("editor", 0);
+  static const char *zEditor = 0;
+  const char *azStdEd[] = {
+    "notepad", "nano", "pico", "jove", "edit", "vi", "vim", "ed"
+  };
+  int i = 0;
+  if( zEditor==0 ){
+    zEditor = find_option("editor",0,1);
+  }
+  if( zEditor==0 ){
+    zEditor = db_get("editor", 0);
+  }
   if( zEditor==0 ){
     zEditor = fossil_getenv("VISUAL");
   }
   if( zEditor==0 ){
     zEditor = fossil_getenv("EDITOR");
   }
-#if defined(_WIN32) || defined(__CYGWIN__)
-  if( zEditor==0 ){
-    zEditor = mprintf("%s\\notepad.exe", fossil_getenv("SYSTEMROOT"));
-#if defined(__CYGWIN__)
-    zEditor = fossil_utf8_to_path(zEditor, 0);
-#endif
+  while( zEditor==0 && i<count(azStdEd) ){
+    if( fossil_app_on_path(azStdEd[i],0) ){
+      zEditor = azStdEd[i];
+    }else{
+      i++;
+    }
   }
-#endif
+  if( zEditor && is_false(zEditor) ) zEditor = 0;
   return zEditor;
 }
 
@@ -888,6 +903,48 @@ void test_random_password(void){
   fossil_free(zPassword);
 }
 
+/*
+** Generate a version 4 ("random"), variant 1 UUID (RFC 9562, Section 5.4).
+**
+** Format: xxxxxxxx-xxxx-Mxxx-Nxxx-xxxxxxxxxxxx
+**           where  M=4  and  N=8, 9, a, or b    (this leaves 122 random bits)
+*/
+char* fossil_generate_uuid() {
+  static const char zDigits[] = "0123456789abcdef";
+  unsigned char aBlob[16];
+  unsigned char zStr[37];
+  unsigned char *p = zStr;
+  int i, k;
+
+  sqlite3_randomness(16, aBlob);
+  aBlob[6] = (aBlob[6]&0x0f) + 0x40; /* Version byte:  0100 xxxx */
+  aBlob[8] = (aBlob[8]&0x3f) + 0x80; /* Variant byte:  1000 xxxx */
+
+  for(i=0, k=0x550; i<16; i++, k=k>>1){
+    if( k&1 ){
+      *p++ = '-';                    /* Add a dash after byte 4, 6, 8, and 12 */
+    }
+    *p++ = zDigits[aBlob[i]>>4];
+    *p++ = zDigits[aBlob[i]&0xf];
+  }
+  *p = 0;
+
+  return fossil_strdup((char*)zStr);
+}
+
+/*
+** COMMAND: test-generate-uuid
+**
+** Usage: %fossil test-generate-uuid
+**
+** Generate a version 4 ("random"), variant 1 UUID (RFC 9562, Section 5.4):
+**
+**     xxxxxxxx-xxxx-Mxxx-Nxxx-xxxxxxxxxxxx  - where  M=4  and  N=8, 9, a, or b
+*/
+void test_generate_uuid(void){
+  fossil_print("%s\n", fossil_generate_uuid());
+}
+
 /*
 ** Return the number of decimal digits in a nonnegative integer.  This is useful
 ** when formatting text.
@@ -898,31 +955,72 @@ int fossil_num_digits(int n){
        : n<10000000 ? 7 : n<100000000 ? 8 : n<1000000000 ? 9 : 10;
 }
 
-#if !defined(_WIN32)
-#if !defined(__DARWIN__) && !defined(__APPLE__) && !defined(__HAIKU__)
 /*
 ** Search for an executable on the PATH environment variable.
 ** Return true (1) if found and false (0) if not found.
+**
+** Print the full pathname of the first location if ePrint==1.  Print
+** all pathnames for the executable if ePrint==2 or more.
 */
-static int binaryOnPath(const char *zBinary){
+int fossil_app_on_path(const char *zBinary, int ePrint){
   const char *zPath = fossil_getenv("PATH");
   char *zFull;
   int i;
   int bExists;
+  int bFound = 0;
   while( zPath && zPath[0] ){
+#ifdef _WIN32
+    while( zPath[0]==';' ) zPath++;
+    for(i=0; zPath[i] && zPath[i]!=';'; i++){}
+    zFull = mprintf("%.*s\\%s.exe", i, zPath, zBinary);
+    bExists = file_access(zFull, R_OK);
+    if( bExists!=0 ){
+      fossil_free(zFull);
+      zFull = mprintf("%.*s\\%s.bat", i, zPath, zBinary);
+      bExists = file_access(zFull, R_OK);
+    }
+#else
     while( zPath[0]==':' ) zPath++;
     for(i=0; zPath[i] && zPath[i]!=':'; i++){}
     zFull = mprintf("%.*s/%s", i, zPath, zBinary);
     bExists = file_access(zFull, X_OK);
+#endif
+    if( bExists==0 && ePrint ){
+      fossil_print("%s\n", zFull);
+    }
     fossil_free(zFull);
-    if( bExists==0 ) return 1;
+    if( bExists==0 ){
+      if( ePrint<2 ) return 1;
+      bFound = 1;
+    }
     zPath += i;
   }
-  return 0;
+  return bFound;
 }
-#endif
-#endif
 
+/*
+** COMMAND: which*
+**
+** Usage: fossil which [-a] NAME ...
+**
+** For each NAME mentioned as an argument, print the first location on the
+** on PATH of the executable with that name.  Or, show all locations on PATH
+** for each argument if the -a option is used.
+**
+** This command is a substitute for the unix "which" command, which is not
+** always available, especially on Windows.
+*/
+void test_app_on_path(void){
+  int i;
+  int ePrint = 1;
+  if( find_option("all","a",0)!=0 ) ePrint = 2;
+  verify_all_options();
+  for(i=2; i<g.argc; i++){
+    if( fossil_app_on_path(g.argv[i], ePrint)==0 ){
+      fossil_print("NOT FOUND: %s\n", g.argv[i]);
+    }
+  }
+}
 
 /*
 ** Return the name of a command that will launch a web-browser.
@@ -941,11 +1039,12 @@ const char *fossil_web_browser(void){
     int i;
     zBrowser = "echo";
     for(i=0; i<count(azBrowserProg); i++){
-      if( binaryOnPath(azBrowserProg[i]) ){
+      if( fossil_app_on_path(azBrowserProg[i],0) ){
         zBrowser = azBrowserProg[i];
         break;
       }
     }
+    zBrowser = mprintf("%s 2>/dev/null", zBrowser);
   }
 #endif
   return zBrowser;
